/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    Pseudo_QC_Binary.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
/* Qore class Qore::<binary> */

qore_classid_t CID_PSEUDOBINARY;
QoreClass* QC_PSEUDOBINARY;

// bool PseudoBinary::empty(){}
static QoreValue PseudoBinary_empty(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 179 "Pseudo_QC_Binary.qpp"
    return b->empty();
}

// int PseudoBinary::find(data bin, softint byte_pos = 0){}
static QoreValue PseudoBinary_find_VDvi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue bin = get_param_value(args, 0);
    int64 byte_pos = HARD_QORE_VALUE_INT(args, 1);
    const BinaryNode* b = v.get<const BinaryNode>();
# 95 "Pseudo_QC_Binary.qpp"
    const char* bin_ptr;
    size_t bin_len;
    q_get_data(bin, bin_ptr, bin_len);

    if (byte_pos < 0) {
        byte_pos = b->size() + byte_pos;
        if (byte_pos < 0) {
            return -1;
        }
    } else if ((size_t)byte_pos >= b->size()) {
        return -1;
    }

    size_t len = b->size() - byte_pos;
    if (!len) {
        return bin_len ? -1 : 0;
    } else if (len < 0) {
        return -1;
    }

    void* ptr = q_memmem(static_cast<const char*>(b->getPtr()) + byte_pos, len, bin_ptr, bin_len);
    if (!ptr) {
        return -1;
    }
    return static_cast<int64>(reinterpret_cast<const char*>(ptr) - reinterpret_cast<const char*>(b->getPtr()));
}

// int PseudoBinary::rfind(data bin, softint byte_pos = -1){}
static QoreValue PseudoBinary_rfind_VDvi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue bin = get_param_value(args, 0);
    int64 byte_pos = HARD_QORE_VALUE_INT(args, 1);
    const BinaryNode* b = v.get<const BinaryNode>();
# 147 "Pseudo_QC_Binary.qpp"
    const char* bin_ptr;
    size_t bin_len;
    q_get_data(bin, bin_ptr, bin_len);

    if (byte_pos < 0) {
        byte_pos = b->size() + byte_pos;
        if (byte_pos < 0) {
            return -1;
        }
    } else if ((size_t)byte_pos >= b->size()) {
        byte_pos = b->size() - 1;
    }

    assert((size_t)byte_pos < b->size());
    void* ptr = q_memrmem(b->getPtr(), byte_pos + 1, bin_ptr, bin_len);
    if (!ptr) {
        return -1;
    }
    return static_cast<int64>(reinterpret_cast<const char*>(ptr) - reinterpret_cast<const char*>(b->getPtr()));
}

// int PseudoBinary::size(){}
static QoreValue PseudoBinary_size(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 68 "Pseudo_QC_Binary.qpp"
    return b->size();
}

// bool PseudoBinary::sizep(){}
static QoreValue PseudoBinary_sizep(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 566 "Pseudo_QC_Binary.qpp"
    return true;
}

// list<binary> PseudoBinary::split(data sep){}
static QoreValue PseudoBinary_split_VD(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue sep = get_param_value(args, 0);
    const BinaryNode* b = v.get<const BinaryNode>();
# 216 "Pseudo_QC_Binary.qpp"
    const char* bin_ptr;
    size_t bin_len;
    q_get_data(sep, bin_ptr, bin_len);

    return split_intern(bin_ptr, bin_len, (const char*)b->getPtr(), b->size(), nullptr);
}

// binary PseudoBinary::substr(softint start){}
static QoreValue PseudoBinary_substr_vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    const BinaryNode* b = v.get<const BinaryNode>();
# 509 "Pseudo_QC_Binary.qpp"
    BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start);
   return b1;
}

// binary PseudoBinary::substr(softint start, softint len){}
static QoreValue PseudoBinary_substr_vivi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    int64 len = HARD_QORE_VALUE_INT(args, 1);
    const BinaryNode* b = v.get<const BinaryNode>();
# 533 "Pseudo_QC_Binary.qpp"
    BinaryNode* b1 = new BinaryNode;
   b->substr(*b1, start, len);
   return b1;
}

// string PseudoBinary::toBase64(softint maxlinelen = -1){}
static QoreValue PseudoBinary_toBase64_vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 0);
    const BinaryNode* b = v.get<const BinaryNode>();
# 466 "Pseudo_QC_Binary.qpp"
    return new QoreStringNode(b, maxlinelen);
}

// string PseudoBinary::toBase64Url(softint maxlinelen = -1){}
static QoreValue PseudoBinary_toBase64Url_vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 0);
    const BinaryNode* b = v.get<const BinaryNode>();
# 486 "Pseudo_QC_Binary.qpp"
    QoreStringNode* rv = new QoreStringNode;
    rv->concatBase64Url(*b);
    return rv;
}

// string PseudoBinary::toHex(){}
static QoreValue PseudoBinary_toHex(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 440 "Pseudo_QC_Binary.qpp"
    QoreStringNode *rv = new QoreStringNode;
   rv->concatHex(b);
   return rv;
}

// string PseudoBinary::toMD5(){}
static QoreValue PseudoBinary_toMD5(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 246 "Pseudo_QC_Binary.qpp"
    DigestHelper dh(*b);
   if (dh.doDigest(MD5_ERR, EVP_md5(), xsink))
      return QoreValue();

   return dh.getString();
}

// string PseudoBinary::toSHA1(){}
static QoreValue PseudoBinary_toSHA1(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 274 "Pseudo_QC_Binary.qpp"
    DigestHelper dh(*b);
   if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink))
      return QoreValue();

   return dh.getString();
}

// string PseudoBinary::toSHA224(){}
static QoreValue PseudoBinary_toSHA224(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 305 "Pseudo_QC_Binary.qpp"
    #if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA224", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// string PseudoBinary::toSHA256(){}
static QoreValue PseudoBinary_toSHA256(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 341 "Pseudo_QC_Binary.qpp"
    #if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA256", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// string PseudoBinary::toSHA384(){}
static QoreValue PseudoBinary_toSHA384(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 377 "Pseudo_QC_Binary.qpp"
    #if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA384", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// string PseudoBinary::toSHA512(){}
static QoreValue PseudoBinary_toSHA512(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 413 "Pseudo_QC_Binary.qpp"
    #if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*b);
   if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA512", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// string PseudoBinary::toString(*string encoding){}
static QoreValue PseudoBinary_toString_Ns(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = get_param_value(args, 0).get<const QoreStringNode>();
    const BinaryNode* b = v.get<const BinaryNode>();
# 552 "Pseudo_QC_Binary.qpp"
    const QoreEncoding *qe = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;
   if (!b->size())
      return new QoreStringNode(qe);
   return new QoreStringNode((const char *)b->getPtr(), b->size(), qe);
}

// int PseudoBinary::typeCode(){}
static QoreValue PseudoBinary_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 53 "Pseudo_QC_Binary.qpp"
    return NT_BINARY;
}

// bool PseudoBinary::val(){}
static QoreValue PseudoBinary_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* b = v.get<const BinaryNode>();
# 197 "Pseudo_QC_Binary.qpp"
    return !b->empty();
}

DLLLOCAL void preinitPseudoBinaryClass() {
    QC_PSEUDOBINARY = new QoreClass("<binary>", "::Qore::<binary>", QDOM_DEFAULT);
    CID_PSEUDOBINARY = QC_PSEUDOBINARY->getID();
    QC_PSEUDOBINARY->setSystem();
}

DLLLOCAL QoreClass* initPseudoBinaryClass() {
    if (!QC_PSEUDOBINARY)
        preinitPseudoBinaryClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDOBINARY->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // bool PseudoBinary::empty(){}
    QC_PSEUDOBINARY->addMethod("empty", (q_method_n_t)PseudoBinary_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoBinary::find(data bin, softint byte_pos = 0){}
    QC_PSEUDOBINARY->addMethod("find", (q_method_n_t)PseudoBinary_find_VDvi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "bin", softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "byte_pos");

    // int PseudoBinary::rfind(data bin, softint byte_pos = -1){}
    QC_PSEUDOBINARY->addMethod("rfind", (q_method_n_t)PseudoBinary_rfind_VDvi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "bin", softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "byte_pos");

    // int PseudoBinary::size(){}
    QC_PSEUDOBINARY->addMethod("size", (q_method_n_t)PseudoBinary_size, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoBinary::sizep(){}
    QC_PSEUDOBINARY->addMethod("sizep", (q_method_n_t)PseudoBinary_sizep, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // list<binary> PseudoBinary::split(data sep){}
    QC_PSEUDOBINARY->addMethod("split", (q_method_n_t)PseudoBinary_split_VD, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(binaryTypeInfo), 1, dataTypeInfo, QORE_PARAM_NO_ARG, "sep");

    // binary PseudoBinary::substr(softint start){}
    QC_PSEUDOBINARY->addMethod("substr", (q_method_n_t)PseudoBinary_substr_vi, Public, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start");

    // binary PseudoBinary::substr(softint start, softint len){}
    QC_PSEUDOBINARY->addMethod("substr", (q_method_n_t)PseudoBinary_substr_vivi, Public, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo, 2, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "start", softBigIntTypeInfo, QORE_PARAM_NO_ARG, "len");

    // string PseudoBinary::toBase64(softint maxlinelen = -1){}
    QC_PSEUDOBINARY->addMethod("toBase64", (q_method_n_t)PseudoBinary_toBase64_vi, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "maxlinelen");

    // string PseudoBinary::toBase64Url(softint maxlinelen = -1){}
    QC_PSEUDOBINARY->addMethod("toBase64Url", (q_method_n_t)PseudoBinary_toBase64Url_vi, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "maxlinelen");

    // string PseudoBinary::toHex(){}
    QC_PSEUDOBINARY->addMethod("toHex", (q_method_n_t)PseudoBinary_toHex, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoBinary::toMD5(){}
    QC_PSEUDOBINARY->addMethod("toMD5", (q_method_n_t)PseudoBinary_toMD5, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoBinary::toSHA1(){}
    QC_PSEUDOBINARY->addMethod("toSHA1", (q_method_n_t)PseudoBinary_toSHA1, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoBinary::toSHA224(){}
    QC_PSEUDOBINARY->addMethod("toSHA224", (q_method_n_t)PseudoBinary_toSHA224, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoBinary::toSHA256(){}
    QC_PSEUDOBINARY->addMethod("toSHA256", (q_method_n_t)PseudoBinary_toSHA256, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoBinary::toSHA384(){}
    QC_PSEUDOBINARY->addMethod("toSHA384", (q_method_n_t)PseudoBinary_toSHA384, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoBinary::toSHA512(){}
    QC_PSEUDOBINARY->addMethod("toSHA512", (q_method_n_t)PseudoBinary_toSHA512, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoBinary::toString(*string encoding){}
    QC_PSEUDOBINARY->addMethod("toString", (q_method_n_t)PseudoBinary_toString_Ns, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // int PseudoBinary::typeCode(){}
    QC_PSEUDOBINARY->addMethod("typeCode", (q_method_n_t)PseudoBinary_typeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoBinary::val(){}
    QC_PSEUDOBINARY->addMethod("val", (q_method_n_t)PseudoBinary_val, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_PSEUDOBINARY;
}
