/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_Number.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/qore_number_private.h"
/* Qore class Qore::<number> */

qore_classid_t CID_PSEUDONUMBER;
QoreClass* QC_PSEUDONUMBER;

// number PseudoNumber::abs(){}
static QoreValue PseudoNumber_abs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 283 "Pseudo_QC_Number.qpp"
    return qore_number_private::doUnary(*n, mpfr_abs);
}

// string PseudoNumber::format(string fmt){}
static QoreValue PseudoNumber_format_Vs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* fmt = HARD_QORE_VALUE_STRING(args, 0);
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 239 "Pseudo_QC_Number.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(fmt->getEncoding()));
   return qore_number_private::get(*n)->format(**rv, *fmt, xsink) ? 0 : rv.release();
}

// string PseudoNumber::format(int prec = 2, string decimal_sep = ".", string thousands_sep = ""){}
static QoreValue PseudoNumber_format_ViVsVs(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 prec = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* decimal_sep = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* thousands_sep = HARD_QORE_VALUE_STRING(args, 2);
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 266 "Pseudo_QC_Number.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode);
   return qore_number_private::get(*n)->format(**rv, prec, *decimal_sep, *thousands_sep, xsink) ? 0 : rv.release();
}

// bool PseudoNumber::infp(){}
static QoreValue PseudoNumber_infp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 170 "Pseudo_QC_Number.qpp"
    return qore_number_private::get(*n)->inf();
}

// bool PseudoNumber::intp(){}
static QoreValue PseudoNumber_intp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 122 "Pseudo_QC_Number.qpp"
    return true;
}

// bool PseudoNumber::nanp(){}
static QoreValue PseudoNumber_nanp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 154 "Pseudo_QC_Number.qpp"
    return qore_number_private::get(*n)->nan();
}

// int PseudoNumber::prec(){}
static QoreValue PseudoNumber_prec(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 198 "Pseudo_QC_Number.qpp"
    return (int64)qore_number_private::get(*n)->getPrec();
}

// int PseudoNumber::sign(){}
static QoreValue PseudoNumber_sign(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 184 "Pseudo_QC_Number.qpp"
    return qore_number_private::get(*n)->sign();
}

// bool PseudoNumber::strp(){}
static QoreValue PseudoNumber_strp(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 138 "Pseudo_QC_Number.qpp"
    return true;
}

// string PseudoNumber::toBase(int base = 10){}
static QoreValue PseudoNumber_toBase_Vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 base = HARD_QORE_VALUE_INT(args, 0);
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 308 "Pseudo_QC_Number.qpp"
    return qore_number_private::toBase(*n, base, xsink);
}

// string PseudoNumber::toString(int fmt){}
static QoreValue PseudoNumber_toString_Vi(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 fmt = HARD_QORE_VALUE_INT(args, 0);
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 209 "Pseudo_QC_Number.qpp"
    QoreStringNode* rv = new QoreStringNode;
   n->toString(*rv, fmt);
   return rv;
}

// int PseudoNumber::typeCode(){}
static QoreValue PseudoNumber_typeCode(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 87 "Pseudo_QC_Number.qpp"
    return NT_NUMBER;
}

// bool PseudoNumber::val(){}
static QoreValue PseudoNumber_val(QoreObject* self, const QoreValue& v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreNumberNode* n = v.get<const QoreNumberNode>();
# 106 "Pseudo_QC_Number.qpp"
    return !n->zero();
}

DLLLOCAL void preinitPseudoNumberClass() {
    QC_PSEUDONUMBER = new QoreClass("<number>", "::Qore::<number>", QDOM_DEFAULT);
    CID_PSEUDONUMBER = QC_PSEUDONUMBER->getID();
    QC_PSEUDONUMBER->setSystem();
}

DLLLOCAL QoreClass* initPseudoNumberClass() {
    if (!QC_PSEUDONUMBER)
        preinitPseudoNumberClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDONUMBER->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // number PseudoNumber::abs(){}
    QC_PSEUDONUMBER->addMethod("abs", (q_method_n_t)PseudoNumber_abs, Public, QCF_CONSTANT, QDOM_DEFAULT, numberTypeInfo);

    // string PseudoNumber::format(string fmt){}
    QC_PSEUDONUMBER->addMethod("format", (q_method_n_t)PseudoNumber_format_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "fmt");

    // string PseudoNumber::format(int prec = 2, string decimal_sep = ".", string thousands_sep = ""){}
    QC_PSEUDONUMBER->addMethod("format", (q_method_n_t)PseudoNumber_format_ViVsVs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 3, bigIntTypeInfo, QoreSimpleValue().assign((int64)2), "prec", stringTypeInfo, QoreSimpleValue().assign(new QoreStringNode(".")), "decimal_sep", stringTypeInfo, QoreSimpleValue().assign(new QoreStringNode("")), "thousands_sep");

    // bool PseudoNumber::infp(){}
    QC_PSEUDONUMBER->addMethod("infp", (q_method_n_t)PseudoNumber_infp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoNumber::intp(){}
    QC_PSEUDONUMBER->addMethod("intp", (q_method_n_t)PseudoNumber_intp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoNumber::nanp(){}
    QC_PSEUDONUMBER->addMethod("nanp", (q_method_n_t)PseudoNumber_nanp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoNumber::prec(){}
    QC_PSEUDONUMBER->addMethod("prec", (q_method_n_t)PseudoNumber_prec, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int PseudoNumber::sign(){}
    QC_PSEUDONUMBER->addMethod("sign", (q_method_n_t)PseudoNumber_sign, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoNumber::strp(){}
    QC_PSEUDONUMBER->addMethod("strp", (q_method_n_t)PseudoNumber_strp, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string PseudoNumber::toBase(int base = 10){}
    QC_PSEUDONUMBER->addMethod("toBase", (q_method_n_t)PseudoNumber_toBase_Vi, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, bigIntTypeInfo, QoreSimpleValue().assign((int64)10), "base");

    // string PseudoNumber::toString(int fmt){}
    QC_PSEUDONUMBER->addMethod("toString", (q_method_n_t)PseudoNumber_toString_Vi, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "fmt");

    // int PseudoNumber::typeCode(){}
    QC_PSEUDONUMBER->addMethod("typeCode", (q_method_n_t)PseudoNumber_typeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoNumber::val(){}
    QC_PSEUDONUMBER->addMethod("val", (q_method_n_t)PseudoNumber_val, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_PSEUDONUMBER;
}

DLLLOCAL void init_QC_Number_constants(QoreNamespace& ns) {
# 36 "Pseudo_QC_Number.qpp"
# 42 "Pseudo_QC_Number.qpp"
    ns.addConstant("NF_Default", ((int64)QORE_NF_DEFAULT));
    ns.addConstant("NF_Raw", ((int64)QORE_NF_RAW));
    ns.addConstant("NF_Scientific", ((int64)QORE_NF_SCIENTIFIC));
    ns.addConstant("SNF_AllDigits", ((int64)QORE_NUM_ALL_DIGITS));

}
