/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractLineIterator.qpp AbstractLineIterator class (interface) definition */
/*
    Qore Programming Language

    Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
/* Qore class Qore::AbstractLineIterator */

qore_classid_t CID_ABSTRACTLINEITERATOR;
QoreClass* QC_ABSTRACTLINEITERATOR;

// list<string> AbstractLineIterator::getSplitLine(string sep, string quote, string eol = "\n", bool trim_unquoted = False){}
static QoreValue AbstractLineIterator_getSplitLine_VsVsVsVb(QoreObject* self, void* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* quote = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* eol = HARD_QORE_VALUE_STRING(args, 2);
    bool trim_unquoted = HARD_QORE_VALUE_BOOL(args, 3);
# 107 "QC_AbstractLineIterator.qpp"
        AbstractIteratorHelper h(xsink, "AbstractLineIterator::getSplitLine() method", self);
    if (*xsink) {
        return QoreValue();
    }
    QoreValue v = h.getValue(xsink);
    if (*xsink) {
        return QoreValue();
    }
    if (v.getType() != NT_STRING) {
        xsink->raiseException("SPLIT-ERROR", "expecting iterator value type 'string'; got type '%s' instead",
            v.getTypeName());
        return QoreValue();
    }
    const QoreStringNode* str = v.get<const QoreStringNode>();
    ReferenceHolder<QoreListNode> rv(split_with_quote(xsink, sep, str, quote, trim_unquoted, &h, eol), xsink);
    if (*xsink) {
        ExceptionSink xsink2;
        ValueHolder line_num(self->evalMethod("index", nullptr, &xsink2), xsink);
        if (line_num->getType() == NT_INT) {
            xsink->appendLastDescription(": error iterating input line " QLLD, (int)line_num->getAsBigInt());
        }
        return QoreValue();
    }
    return rv.release();
}

DLLLOCAL void preinitAbstractLineIteratorClass() {
    QC_ABSTRACTLINEITERATOR = new QoreClass("AbstractLineIterator", "::Qore::AbstractLineIterator", QDOM_DEFAULT);
    CID_ABSTRACTLINEITERATOR = QC_ABSTRACTLINEITERATOR->getID();
    QC_ABSTRACTLINEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initAbstractLineIteratorClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTLINEITERATOR)
        preinitAbstractLineIteratorClass();

    // set parent class
    assert(QC_ABSTRACTITERATOR);
    QC_ABSTRACTLINEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

    // string AbstractLineIterator::getLine();
    QC_ABSTRACTLINEITERATOR->addAbstractMethod("getLine", Public, QCF_NO_FLAGS, stringTypeInfo);

    // list<string> AbstractLineIterator::getSplitLine(string sep, string quote, string eol = "\n", bool trim_unquoted = False){}
    QC_ABSTRACTLINEITERATOR->addMethod("getSplitLine", (q_method_n_t)AbstractLineIterator_getSplitLine_VsVsVsVb, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 4, stringTypeInfo, QORE_PARAM_NO_ARG, "sep", stringTypeInfo, QORE_PARAM_NO_ARG, "quote", stringTypeInfo, QoreSimpleValue().assign(new QoreStringNode("\n")), "eol", boolTypeInfo, QoreSimpleValue().assign(false), "trim_unquoted");

    // string AbstractLineIterator::getValue();
    QC_ABSTRACTLINEITERATOR->addAbstractMethod("getValue", Public, QCF_NO_FLAGS, stringTypeInfo);

    // int AbstractLineIterator::index();
    QC_ABSTRACTLINEITERATOR->addAbstractMethod("index", Public, QCF_NO_FLAGS, bigIntTypeInfo);

    // bool AbstractLineIterator::next();
    QC_ABSTRACTLINEITERATOR->addAbstractMethod("next", Public, QCF_NO_FLAGS, boolTypeInfo);

    // bool AbstractLineIterator::valid();
    QC_ABSTRACTLINEITERATOR->addAbstractMethod("valid", Public, QCF_NO_FLAGS, boolTypeInfo);

    return QC_ABSTRACTLINEITERATOR;
}
