/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_Breakpoint.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Breakpoint.h"
#include "qore/intern/QC_Program.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/AbstractStatement.h"
/* Qore class Qore::Breakpoint */

qore_classid_t CID_BREAKPOINT;
QoreClass* QC_BREAKPOINT;

// nothing Breakpoint::addThreadId(int tid){}
static QoreValue Breakpoint_addThreadId_Vi(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 tid = HARD_QORE_VALUE_INT(args, 0);
# 242 "QC_Breakpoint.qpp"
    bkpt->addThreadId(tid, xsink);
    return QoreValue();
}

// nothing Breakpoint::assignProgram(ProgramControl pgm){}
static QoreValue Breakpoint_assignProgram_C14ProgramControl(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(pgm, QoreProgram, args, 0, CID_PROGRAMCONTROL, "Breakpoint::assignProgram()", "ProgramControl", xsink);
    if (*xsink)
        return 0;
# 153 "QC_Breakpoint.qpp"
    ReferenceHolder<QoreProgram> holder(pgm, xsink);
   bkpt->assignProgram(pgm, xsink);
    return QoreValue();
}

// nothing Breakpoint::assignStatement(int statementId){}
static QoreValue Breakpoint_assignStatement_Vi(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 statementId = HARD_QORE_VALUE_INT(args, 0);
# 183 "QC_Breakpoint.qpp"
    AbstractStatement *s = bkpt->resolveStatementId(statementId, xsink);
   if (!xsink->isEvent()) {
      bkpt->assignStatement(s, xsink);
   }
    return QoreValue();
}

// nothing Breakpoint::clearThreadIds(){}
static QoreValue Breakpoint_clearThreadIds(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 266 "QC_Breakpoint.qpp"
    bkpt->clearThreadIds(xsink);
    return QoreValue();
}

// Breakpoint::constructor() {}
static void Breakpoint_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 85 "QC_Breakpoint.qpp"
QoreBreakpoint *qb = new QoreBreakpoint;
   qb->setQoreObject(self);
   self->setPrivate(CID_BREAKPOINT, qb);
}

// Breakpoint::copy() {}
static void Breakpoint_copy(QoreObject* self, QoreObject* old, QoreBreakpoint* bkpt, ExceptionSink* xsink) {
# 94 "QC_Breakpoint.qpp"
QoreBreakpoint *qb = new QoreBreakpoint(*bkpt);
   qb->setQoreObject(self);
   self->setPrivate(CID_BREAKPOINT, qb);
}

// Breakpoint::destructor() {}
static void Breakpoint_destructor(QoreObject* self, QoreBreakpoint* bkpt, ExceptionSink* xsink) {
# 104 "QC_Breakpoint.qpp"
bkpt->setQoreObject(0);
   bkpt->deref();
}

// int Breakpoint::getBreakpointId(){}
static QoreValue Breakpoint_getBreakpointId(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 273 "QC_Breakpoint.qpp"
    return bkpt->getBreakpointId();
}

// bool Breakpoint::getEnabled(){}
static QoreValue Breakpoint_getEnabled(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 121 "QC_Breakpoint.qpp"
    return bkpt->enabled;
}

// int Breakpoint::getPolicy(){}
static QoreValue Breakpoint_getPolicy(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 144 "QC_Breakpoint.qpp"
    return (int)bkpt->policy;
}

// ProgramControl Breakpoint::getProgram(){}
static QoreValue Breakpoint_getProgram(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 171 "QC_Breakpoint.qpp"
    QoreProgram* pgm = bkpt->getProgram();
   if (!pgm) {
      return xsink->raiseException("BREAKPOINT-ERROR", "Program is not assigned");
   }
   return QoreProgram::getQoreObject(pgm);
}

// list<int> Breakpoint::getStatementIds(){}
static QoreValue Breakpoint_getStatementIds(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 205 "QC_Breakpoint.qpp"
    return bkpt->getStatementIds(xsink);
}

// list<int> Breakpoint::getThreadIds(){}
static QoreValue Breakpoint_getThreadIds(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 213 "QC_Breakpoint.qpp"
    TidList_t tl;
    bkpt->getThreadIds(tl, xsink);
    ReferenceHolder<QoreListNode> l(new QoreListNode(bigIntTypeInfo), xsink);
    for (TidList_t::iterator it = tl.begin(); it != tl.end(); ++it) {
        (*l)->push(*it, xsink);
    }
    return l.release();
}

// bool Breakpoint::isThreadId(softint tid){}
static QoreValue Breakpoint_isThreadId_vi(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 tid = HARD_QORE_VALUE_INT(args, 0);
# 258 "QC_Breakpoint.qpp"
    return bkpt->isThreadId(tid, xsink);
}

// nothing Breakpoint::removeThreadId(int tid){}
static QoreValue Breakpoint_removeThreadId_Vi(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 tid = HARD_QORE_VALUE_INT(args, 0);
# 250 "QC_Breakpoint.qpp"
    bkpt->removeThreadId(tid, xsink);
    return QoreValue();
}

// nothing Breakpoint::setEnabled(bool enabled){}
static QoreValue Breakpoint_setEnabled_Vb(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool enabled = HARD_QORE_VALUE_BOOL(args, 0);
# 113 "QC_Breakpoint.qpp"
    bkpt->enabled = enabled;
    return QoreValue();
}

// nothing Breakpoint::setPolicy(softint policy){}
static QoreValue Breakpoint_setPolicy_vi(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 policy = HARD_QORE_VALUE_INT(args, 0);
# 129 "QC_Breakpoint.qpp"
    switch (policy) {
   case BKP_PO_NONE:
   case BKP_PO_ACCEPT:
   case BKP_PO_REJECT:
      bkpt->policy = (BreakpointPolicy)policy;
      break;
   default:
      xsink->raiseException("BREAKPOINT-ERROR", "wrong policy value");
   }
    return QoreValue();
}

// nothing Breakpoint::setThreadIds(softlist<softint> tidList){}
static QoreValue Breakpoint_setThreadIds_C17softlist_softint_(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* tidList = HARD_QORE_VALUE_LIST(args, 0);
# 227 "QC_Breakpoint.qpp"
    TidList_t tl;
    for (size_t i = 0, e = tidList->size(); i < e; ++i) {
        QoreValue n = tidList->retrieveEntry(i);
        if (!n.isNothing()) {
            tl.push_back((int)n.getAsBigInt());
        }
    }
    bkpt->setThreadIds(tl, xsink);  // TODO: or just pass tidList ???
    return QoreValue();
}

// nothing Breakpoint::unassignProgram(){}
static QoreValue Breakpoint_unassignProgram(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 162 "QC_Breakpoint.qpp"
    bkpt->assignProgram(nullptr, xsink);
    return QoreValue();
}

// nothing Breakpoint::unassignStatement(int statementId){}
static QoreValue Breakpoint_unassignStatement_Vi(QoreObject* self, QoreBreakpoint* bkpt, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 statementId = HARD_QORE_VALUE_INT(args, 0);
# 194 "QC_Breakpoint.qpp"
    AbstractStatement *s = bkpt->resolveStatementId(statementId, xsink);
   if (!xsink->isEvent()) {
      bkpt->unassignStatement(s, xsink);
   }
    return QoreValue();
}

// static Breakpoint Breakpoint::resolveBreakpointId(int breakpointId){}
static QoreValue static_Breakpoint_resolveBreakpointId_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 breakpointId = HARD_QORE_VALUE_INT(args, 0);
# 282 "QC_Breakpoint.qpp"
QoreBreakpoint* b = QoreBreakpoint::resolveBreakpointId(breakpointId);
   if (!b)
      return xsink->raiseException("BREAKPOINT-ERROR", "wrong breakpointId (%d)", breakpointId);
   return b->getQoreObject();
}

DLLLOCAL void preinitBreakpointClass() {
    QC_BREAKPOINT = new QoreClass("Breakpoint", "::Qore::Breakpoint", QDOM_DEBUGGER);
    CID_BREAKPOINT = QC_BREAKPOINT->getID();
    QC_BREAKPOINT->setSystem();
}

DLLLOCAL QoreClass* initBreakpointClass(QoreNamespace& ns) {
    if (!QC_BREAKPOINT)
        preinitBreakpointClass();

    // nothing Breakpoint::addThreadId(int tid){}
    QC_BREAKPOINT->addMethod("addThreadId", (q_method_n_t)Breakpoint_addThreadId_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "tid");

    // nothing Breakpoint::assignProgram(ProgramControl pgm){}
    QC_BREAKPOINT->addMethod("assignProgram", (q_method_n_t)Breakpoint_assignProgram_C14ProgramControl, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_PROGRAMCONTROL->getTypeInfo(), QORE_PARAM_NO_ARG, "pgm");

    // nothing Breakpoint::assignStatement(int statementId){}
    QC_BREAKPOINT->addMethod("assignStatement", (q_method_n_t)Breakpoint_assignStatement_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "statementId");

    // nothing Breakpoint::clearThreadIds(){}
    QC_BREAKPOINT->addMethod("clearThreadIds", (q_method_n_t)Breakpoint_clearThreadIds, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // Breakpoint::constructor() {}
    QC_BREAKPOINT->addConstructor(Breakpoint_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // Breakpoint::copy() {}
    QC_BREAKPOINT->setCopy((q_copy_t)Breakpoint_copy);

    // Breakpoint::destructor() {}
    QC_BREAKPOINT->setDestructor((q_destructor_t)Breakpoint_destructor);

    // int Breakpoint::getBreakpointId(){}
    QC_BREAKPOINT->addMethod("getBreakpointId", (q_method_n_t)Breakpoint_getBreakpointId, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // bool Breakpoint::getEnabled(){}
    QC_BREAKPOINT->addMethod("getEnabled", (q_method_n_t)Breakpoint_getEnabled, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // int Breakpoint::getPolicy(){}
    QC_BREAKPOINT->addMethod("getPolicy", (q_method_n_t)Breakpoint_getPolicy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // ProgramControl Breakpoint::getProgram(){}
    QC_BREAKPOINT->addMethod("getProgram", (q_method_n_t)Breakpoint_getProgram, Public, QCF_RET_VALUE_ONLY, QDOM_DEBUGGER, QC_PROGRAMCONTROL->getTypeInfo());

    // list<int> Breakpoint::getStatementIds(){}
    QC_BREAKPOINT->addMethod("getStatementIds", (q_method_n_t)Breakpoint_getStatementIds, Public, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(bigIntTypeInfo));

    // list<int> Breakpoint::getThreadIds(){}
    QC_BREAKPOINT->addMethod("getThreadIds", (q_method_n_t)Breakpoint_getThreadIds, Public, QCF_NO_FLAGS, QDOM_DEFAULT, qore_get_complex_list_type(bigIntTypeInfo));

    // bool Breakpoint::isThreadId(softint tid){}
    QC_BREAKPOINT->addMethod("isThreadId", (q_method_n_t)Breakpoint_isThreadId_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "tid");

    // nothing Breakpoint::removeThreadId(int tid){}
    QC_BREAKPOINT->addMethod("removeThreadId", (q_method_n_t)Breakpoint_removeThreadId_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "tid");

    // nothing Breakpoint::setEnabled(bool enabled){}
    QC_BREAKPOINT->addMethod("setEnabled", (q_method_n_t)Breakpoint_setEnabled_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, QORE_PARAM_NO_ARG, "enabled");

    // nothing Breakpoint::setPolicy(softint policy){}
    QC_BREAKPOINT->addMethod("setPolicy", (q_method_n_t)Breakpoint_setPolicy_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, QORE_PARAM_NO_ARG, "policy");

    // nothing Breakpoint::setThreadIds(softlist<softint> tidList){}
    QC_BREAKPOINT->addMethod("setThreadIds", (q_method_n_t)Breakpoint_setThreadIds_C17softlist_softint_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, qore_get_complex_softlist_type(softBigIntTypeInfo), QORE_PARAM_NO_ARG, "tidList");

    // nothing Breakpoint::unassignProgram(){}
    QC_BREAKPOINT->addMethod("unassignProgram", (q_method_n_t)Breakpoint_unassignProgram, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Breakpoint::unassignStatement(int statementId){}
    QC_BREAKPOINT->addMethod("unassignStatement", (q_method_n_t)Breakpoint_unassignStatement_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "statementId");

    // static Breakpoint Breakpoint::resolveBreakpointId(int breakpointId){}
    QC_BREAKPOINT->addStaticMethod("resolveBreakpointId", (q_func_n_t)static_Breakpoint_resolveBreakpointId_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_BREAKPOINT->getTypeInfo(), 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "breakpointId");
# 40 "QC_Breakpoint.qpp"
# 50 "QC_Breakpoint.qpp"
# 53 "QC_Breakpoint.qpp"
    ns.addConstant("BreakpointPolicyAccept", ((int64)BKP_PO_ACCEPT));
    ns.addConstant("BreakpointPolicyNone", ((int64)BKP_PO_NONE));
    ns.addConstant("BreakpointPolicyReject", ((int64)BKP_PO_REJECT));

    return QC_BREAKPOINT;
}
