/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_DataLineIterator.qpp DataLineIterator class definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <cerrno>
#include <cstring>

#include "qore/Qore.h"
#include "qore/intern/DataLineIterator.h"
/* Qore class Qore::DataLineIterator */

qore_classid_t CID_DATALINEITERATOR;
QoreClass* QC_DATALINEITERATOR;

// DataLineIterator::constructor(string str, *string eol, bool trim = True) {}
static void DataLineIterator_constructor_VsNsVb(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* eol = get_param_value(args, 1).get<const QoreStringNode>();
    bool trim = HARD_QORE_VALUE_BOOL(args, 2);
# 69 "QC_DataLineIterator.qpp"
if (eol && eol->empty())
      eol = 0;

   SimpleRefHolder<DataLineIterator> dli(new DataLineIterator(xsink, str, eol, trim));
   if (*xsink)
      return;

   self->setPrivate(CID_DATALINEITERATOR, dli.release());
}

// DataLineIterator::copy() {}
static void DataLineIterator_copy(QoreObject* self, QoreObject* old, DataLineIterator* i, ExceptionSink* xsink) {
# 86 "QC_DataLineIterator.qpp"
SimpleRefHolder<DataLineIterator> dli(new DataLineIterator(xsink, *i));

   if (!*xsink)
      self->setPrivate(CID_DATALINEITERATOR, dli.release());
}

// string DataLineIterator::getEncoding(){}
static QoreValue DataLineIterator_getEncoding(QoreObject* self, DataLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 187 "QC_DataLineIterator.qpp"
    return new QoreStringNode(i->getEncoding()->getCode());
}

// string DataLineIterator::getLine(){}
static QoreValue DataLineIterator_getLine(QoreObject* self, DataLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 148 "QC_DataLineIterator.qpp"
    return i->checkValid(xsink) ? 0 : i->getValue();
}

// string DataLineIterator::getValue(){}
static QoreValue DataLineIterator_getValue(QoreObject* self, DataLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 129 "QC_DataLineIterator.qpp"
    return i->checkValid(xsink) ? 0 : i->getValue();
}

// int DataLineIterator::index(){}
static QoreValue DataLineIterator_index(QoreObject* self, DataLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 175 "QC_DataLineIterator.qpp"
    return i->index();
}

// bool DataLineIterator::next(){}
static QoreValue DataLineIterator_next(QoreObject* self, DataLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 108 "QC_DataLineIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next(xsink);
}

// nothing DataLineIterator::reset(){}
static QoreValue DataLineIterator_reset(QoreObject* self, DataLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 201 "QC_DataLineIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   i->reset(xsink);
    return QoreValue();
}

// bool DataLineIterator::valid(){}
static QoreValue DataLineIterator_valid(QoreObject* self, DataLineIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 161 "QC_DataLineIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitDataLineIteratorClass() {
    QC_DATALINEITERATOR = new QoreClass("DataLineIterator", "::Qore::DataLineIterator", QDOM_DEFAULT);
    CID_DATALINEITERATOR = QC_DATALINEITERATOR->getID();
    QC_DATALINEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initDataLineIteratorClass(QoreNamespace& ns) {
    if (!QC_DATALINEITERATOR)
        preinitDataLineIteratorClass();

    // set parent class
    assert(QC_ABSTRACTLINEITERATOR);
    QC_DATALINEITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTLINEITERATOR);

    // DataLineIterator::constructor(string str, *string eol, bool trim = True) {}
    QC_DATALINEITERATOR->addConstructor(DataLineIterator_constructor_VsNsVb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "str", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "eol", boolTypeInfo, QoreSimpleValue().assign(true), "trim");

    // DataLineIterator::copy() {}
    QC_DATALINEITERATOR->setCopy((q_copy_t)DataLineIterator_copy);

    // string DataLineIterator::getEncoding(){}
    QC_DATALINEITERATOR->addMethod("getEncoding", (q_method_n_t)DataLineIterator_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string DataLineIterator::getLine(){}
    QC_DATALINEITERATOR->addMethod("getLine", (q_method_n_t)DataLineIterator_getLine, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string DataLineIterator::getValue(){}
    QC_DATALINEITERATOR->addMethod("getValue", (q_method_n_t)DataLineIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // int DataLineIterator::index(){}
    QC_DATALINEITERATOR->addMethod("index", (q_method_n_t)DataLineIterator_index, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool DataLineIterator::next(){}
    QC_DATALINEITERATOR->addMethod("next", (q_method_n_t)DataLineIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing DataLineIterator::reset(){}
    QC_DATALINEITERATOR->addMethod("reset", (q_method_n_t)DataLineIterator_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool DataLineIterator::valid(){}
    QC_DATALINEITERATOR->addMethod("valid", (q_method_n_t)DataLineIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_DATALINEITERATOR;
}
