/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_EncodingConversionOutputStream.qpp EncodingConversionOutputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/EncodingConversionOutputStream.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"
/* Qore class Qore::EncodingConversionOutputStream */

qore_classid_t CID_ENCODINGCONVERSIONOUTPUTSTREAM;
QoreClass* QC_ENCODINGCONVERSIONOUTPUTSTREAM;

// nothing EncodingConversionOutputStream::close(){}
static QoreValue EncodingConversionOutputStream_close(QoreObject* self, EncodingConversionOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 80 "QC_EncodingConversionOutputStream.qpp"
    os->closeHelper(xsink);
    return QoreValue();
}

// EncodingConversionOutputStream::constructor(Qore::OutputStream os, string sourceEncoding, string destEncoding) {}
static void EncodingConversionOutputStream_constructor_C12OutputStreamVsVs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 0, CID_OUTPUTSTREAM, "EncodingConversionOutputStream::constructor()", "Qore::OutputStream", xsink);
    if (*xsink)
        return;
    const QoreStringNode* sourceEncoding = HARD_QORE_VALUE_STRING(args, 1);
    const QoreStringNode* destEncoding = HARD_QORE_VALUE_STRING(args, 2);
# 65 "QC_EncodingConversionOutputStream.qpp"
SimpleRefHolder<EncodingConversionOutputStream> ecos(new EncodingConversionOutputStream(
         os, QEM.findCreate(sourceEncoding), QEM.findCreate(destEncoding), xsink));
   self->setPrivate(CID_ENCODINGCONVERSIONOUTPUTSTREAM, ecos.release());
   qore_object_private* o = qore_object_private::get(*self);
   const qore_class_private* cls = qore_class_private::get(*QC_ENCODINGCONVERSIONOUTPUTSTREAM);
   o->setValueIntern(cls, "os", static_cast<QoreObject *>(obj_os->refSelf()), xsink);
}

// nothing EncodingConversionOutputStream::write(binary data){}
static QoreValue EncodingConversionOutputStream_write_Vy(QoreObject* self, EncodingConversionOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 98 "QC_EncodingConversionOutputStream.qpp"
    os->writeHelper(data, xsink);
    return QoreValue();
}

DLLLOCAL void preinitEncodingConversionOutputStreamClass() {
    QC_ENCODINGCONVERSIONOUTPUTSTREAM = new QoreClass("EncodingConversionOutputStream", "::Qore::EncodingConversionOutputStream", QDOM_DEFAULT);
    CID_ENCODINGCONVERSIONOUTPUTSTREAM = QC_ENCODINGCONVERSIONOUTPUTSTREAM->getID();
    QC_ENCODINGCONVERSIONOUTPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initEncodingConversionOutputStreamClass(QoreNamespace& ns) {
    if (!QC_ENCODINGCONVERSIONOUTPUTSTREAM)
        preinitEncodingConversionOutputStreamClass();

    // set parent class
    assert(QC_OUTPUTSTREAM);
    QC_ENCODINGCONVERSIONOUTPUTSTREAM->addBuiltinVirtualBaseClass(QC_OUTPUTSTREAM);

    // private:internal members
    QC_ENCODINGCONVERSIONOUTPUTSTREAM->addMember("os", Internal, QC_OUTPUTSTREAM->getTypeInfo());

    QC_ENCODINGCONVERSIONOUTPUTSTREAM->setFinal();

    // nothing EncodingConversionOutputStream::close(){}
    QC_ENCODINGCONVERSIONOUTPUTSTREAM->addMethod("close", (q_method_n_t)EncodingConversionOutputStream_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // EncodingConversionOutputStream::constructor(Qore::OutputStream os, string sourceEncoding, string destEncoding) {}
    QC_ENCODINGCONVERSIONOUTPUTSTREAM->addConstructor(EncodingConversionOutputStream_constructor_C12OutputStreamVsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os", stringTypeInfo, QORE_PARAM_NO_ARG, "sourceEncoding", stringTypeInfo, QORE_PARAM_NO_ARG, "destEncoding");

    // nothing EncodingConversionOutputStream::write(binary data){}
    QC_ENCODINGCONVERSIONOUTPUTSTREAM->addMethod("write", (q_method_n_t)EncodingConversionOutputStream_write_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    return QC_ENCODINGCONVERSIONOUTPUTSTREAM;
}
