/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_FilePollOperation.qpp

    FilePollOperation class definition

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore/Qore.h"
#include "qore/intern/QC_FilePollOperation.h"
/* Qore class Qore::FilePollOperation */

qore_classid_t CID_FILEPOLLOPERATION;
QoreClass* QC_FILEPOLLOPERATION;

// FilePollOperation::constructor(ReadOnlyFile file, string path, bool to_string, int num_bytes = -1) {}
static void FilePollOperation_constructor_C12ReadOnlyFileVsVbVi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(file, File, args, 0, CID_READONLYFILE, "FilePollOperation::constructor()", "ReadOnlyFile", xsink);
    if (*xsink)
        return;
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 1);
    bool to_string = HARD_QORE_VALUE_BOOL(args, 2);
    int64 num_bytes = HARD_QORE_VALUE_INT(args, 3);
# 56 "QC_FilePollOperation.qpp"
ReferenceHolder<File> holder(file, xsink);
    TempEncodingHelper npath(path, QCS_DEFAULT, xsink);
    if (*xsink) {
        return;
    }
    ReferenceHolder<SocketPollOperationBase> poller(new FileReadPollOperation(xsink, holder.release(), npath->c_str(),
        num_bytes, to_string), xsink);
    if (*xsink) {
        return;
    }

    poller->setSelf(self);
    self->setPrivate(CID_FILEPOLLOPERATION, poller.release());
    if (!*xsink) {
        self->setValue("sock", obj_file->objectRefSelf(), xsink);
        self->setValue("goal", new QoreStringNode("read-done"), xsink);
    }
}

// *hash<SocketPollInfo> FilePollOperation::continuePoll(){}
static QoreValue FilePollOperation_continuePoll(QoreObject* self, SocketPollOperationBase* op, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 97 "QC_FilePollOperation.qpp"
    return op->continuePoll(xsink);
}

// FilePollOperation::copy() {}
static void FilePollOperation_copy(QoreObject* self, QoreObject* old, SocketPollOperationBase* op, ExceptionSink* xsink) {
# 87 "QC_FilePollOperation.qpp"
xsink->raiseException("FILEPOLLOPERATION-COPY-ERROR", "objects of this class cannot be copied");
}

// FilePollOperation::destructor() {}
static void FilePollOperation_destructor(QoreObject* self, SocketPollOperationBase* op, ExceptionSink* xsink) {
# 79 "QC_FilePollOperation.qpp"
op->deref(xsink);
}

DLLLOCAL void preinitFilePollOperationClass() {
    QC_FILEPOLLOPERATION = new QoreClass("FilePollOperation", "::Qore::FilePollOperation", QDOM_DEFAULT);
    CID_FILEPOLLOPERATION = QC_FILEPOLLOPERATION->getID();
    QC_FILEPOLLOPERATION->setSystem();
}

DLLLOCAL QoreClass* initFilePollOperationClass(QoreNamespace& ns) {
    if (!QC_FILEPOLLOPERATION)
        preinitFilePollOperationClass();

    // set parent class
    assert(QC_SOCKETPOLLOPERATIONBASE);
    QC_FILEPOLLOPERATION->addBuiltinVirtualBaseClass(QC_SOCKETPOLLOPERATIONBASE);

    // FilePollOperation::constructor(ReadOnlyFile file, string path, bool to_string, int num_bytes = -1) {}
    QC_FILEPOLLOPERATION->addConstructor(FilePollOperation_constructor_C12ReadOnlyFileVsVbVi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 4, QC_READONLYFILE->getTypeInfo(), QORE_PARAM_NO_ARG, "file", stringTypeInfo, QORE_PARAM_NO_ARG, "path", boolTypeInfo, QORE_PARAM_NO_ARG, "to_string", bigIntTypeInfo, QoreSimpleValue().assign((int64)-1), "num_bytes");

    // *hash<SocketPollInfo> FilePollOperation::continuePoll(){}
    QC_FILEPOLLOPERATION->addMethod("continuePoll", (q_method_n_t)FilePollOperation_continuePoll, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclSocketPollInfo->getTypeInfo(true));

    // FilePollOperation::copy() {}
    QC_FILEPOLLOPERATION->setCopy((q_copy_t)FilePollOperation_copy);

    // FilePollOperation::destructor() {}
    QC_FILEPOLLOPERATION->setDestructor((q_destructor_t)FilePollOperation_destructor);

    return QC_FILEPOLLOPERATION;
}
