/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    QC_FtpClient.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore/Qore.h"
#include "qore/intern/ssl_constants.h"
#include "qore/intern/QC_FtpClient.h"
#include "qore/intern/QC_Queue.h"
TypedHashDecl* init_hashdecl_FtpResponseInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("FtpResponseInfo", "::Qore::FtpResponseInfo");
    hd->addMember("code", bigIntTypeInfo, QoreValue());
    hd->addMember("msg", stringTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::FtpClient */

qore_classid_t CID_FTPCLIENT;
QoreClass* QC_FTPCLIENT;

// nothing FtpClient::clearStats(){}
static QoreValue FtpClient_clearStats(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1252 "QC_FtpClient.qpp"
    f->clearStats();
    return QoreValue();
}

// nothing FtpClient::clearWarningQueue(){}
static QoreValue FtpClient_clearWarningQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1171 "QC_FtpClient.qpp"
    f->clearWarningQueue(xsink);
    return QoreValue();
}

// nothing FtpClient::connect(){}
static QoreValue FtpClient_connect(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 178 "QC_FtpClient.qpp"
    f->connect(xsink);
    return QoreValue();
}

// FtpClient::constructor() {}
static void FtpClient_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 103 "QC_FtpClient.qpp"
self->setPrivate(CID_FTPCLIENT, new QoreFtpClientClass);
}

// FtpClient::constructor(string url) {}
static void FtpClient_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 120 "QC_FtpClient.qpp"
ReferenceHolder<QoreFtpClientClass> f(new QoreFtpClientClass(url, xsink), xsink);
    if (*xsink) {
        return;
    }
    const char* url_path = f->getUrlPath();
    if (url_path) {
        ValueHolder val(new QoreStringNode(url_path), xsink);
        if (self->setMemberValue("path", QC_FTPCLIENT, *val, xsink)) {
            return;
        }
    }

    self->setPrivate(CID_FTPCLIENT, f.release());
}

// FtpClient::copy() {}
static void FtpClient_copy(QoreObject* self, QoreObject* old, QoreFtpClientClass* f, ExceptionSink* xsink) {
# 140 "QC_FtpClient.qpp"
xsink->raiseException("FTPCLIENT-COPY-ERROR", "FtpClient objects cannot be copied");
}

// nothing FtpClient::cwd(string path){}
static QoreValue FtpClient_cwd_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 322 "QC_FtpClient.qpp"
    f->cwd(path->c_str(), xsink);
    return QoreValue();
}

// nothing FtpClient::del(string remote_path){}
static QoreValue FtpClient_del_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 562 "QC_FtpClient.qpp"
    f->del(remote_path->c_str(), xsink);
    return QoreValue();
}

// FtpClient::destructor() {}
static void FtpClient_destructor(QoreObject* self, QoreFtpClientClass* f, ExceptionSink* xsink) {
# 154 "QC_FtpClient.qpp"
// have to clear callbacks before destroying
   f->cleanup(xsink);
   f->deref(xsink);
}

// nothing FtpClient::disconnect(){}
static QoreValue FtpClient_disconnect(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 191 "QC_FtpClient.qpp"
    f->disconnect();
    return QoreValue();
}

// nothing FtpClient::get(string remote_path, *string local_path){}
static QoreValue FtpClient_get_VsNs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* local_path = get_param_value(args, 1).get<const QoreStringNode>();
# 420 "QC_FtpClient.qpp"
    f->get(remote_path->c_str(), local_path ? local_path->c_str() : 0, xsink);
    return QoreValue();
}

// nothing FtpClient::get(string remote_path, Qore::OutputStream os){}
static QoreValue FtpClient_get_VsC12OutputStream(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 1, CID_OUTPUTSTREAM, "FtpClient::get()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
# 445 "QC_FtpClient.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
    f->get(remote_path->c_str(), os, xsink);
    return QoreValue();
}

// binary FtpClient::getAsBinary(string remote_path){}
static QoreValue FtpClient_getAsBinary_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 542 "QC_FtpClient.qpp"
    return f->getAsBinary(remote_path->c_str(), xsink);
}

// string FtpClient::getAsString(string remote_path, *string encoding){}
static QoreValue FtpClient_getAsString_VsNs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* encoding = get_param_value(args, 1).get<const QoreStringNode>();
# 517 "QC_FtpClient.qpp"
    return f->getAsString(xsink, remote_path->c_str(), encoding ? QEM.findCreate(encoding) : QCS_DEFAULT);
}

// hash<auto> FtpClient::getControlPeerInfo(bool host_lookup = True){}
static QoreValue FtpClient_getControlPeerInfo_Vb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 794 "QC_FtpClient.qpp"
    return f->getControlPeerInfo(xsink, host_lookup);
}

// hash<auto> FtpClient::getControlSocketInfo(bool host_lookup = True){}
static QoreValue FtpClient_getControlSocketInfo_Vb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 752 "QC_FtpClient.qpp"
    return f->getControlSocketInfo(xsink, host_lookup);
}

// hash<auto> FtpClient::getDataPeerInfo(bool host_lookup = True){}
static QoreValue FtpClient_getDataPeerInfo_Vb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 815 "QC_FtpClient.qpp"
    return f->getDataPeerInfo(xsink, host_lookup);
}

// hash<auto> FtpClient::getDataSocketInfo(bool host_lookup = True){}
static QoreValue FtpClient_getDataSocketInfo_Vb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool host_lookup = HARD_QORE_VALUE_BOOL(args, 0);
# 773 "QC_FtpClient.qpp"
    return f->getDataSocketInfo(xsink, host_lookup);
}

// *string FtpClient::getHostName(){}
static QoreValue FtpClient_getHostName(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 842 "QC_FtpClient.qpp"
    const char *h = f->getHostName();
    return h ? new QoreStringNode(h) : nullptr;
}

// string FtpClient::getMode(){}
static QoreValue FtpClient_getMode(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1043 "QC_FtpClient.qpp"
    return new QoreStringNode(f->getMode());
}

// int FtpClient::getNetworkFamily(){}
static QoreValue FtpClient_getNetworkFamily(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 717 "QC_FtpClient.qpp"
    return f->getNetworkFamily();
}

// *string FtpClient::getPassword(){}
static QoreValue FtpClient_getPassword(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 828 "QC_FtpClient.qpp"
    const char *p = f->getPassword();
    return p ? new QoreStringNode(p) : nullptr;
}

// int FtpClient::getPort(){}
static QoreValue FtpClient_getPort(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 856 "QC_FtpClient.qpp"
    return f->getPort();
}

// *string FtpClient::getSSLCipherName(){}
static QoreValue FtpClient_getSSLCipherName(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 958 "QC_FtpClient.qpp"
    const char *str = f->getSSLCipherName();
    return str ? new QoreStringNode(str) : nullptr;
}

// *string FtpClient::getSSLCipherVersion(){}
static QoreValue FtpClient_getSSLCipherVersion(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 972 "QC_FtpClient.qpp"
    const char *str = f->getSSLCipherVersion();
    return str ? new QoreStringNode(str) : nullptr;
}

// int FtpClient::getTimeout(){}
static QoreValue FtpClient_getTimeout(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1261 "QC_FtpClient.qpp"
    return f->getTimeout();
}

// string FtpClient::getURL(){}
static QoreValue FtpClient_getURL(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 869 "QC_FtpClient.qpp"
    return f->getURL();
}

// hash FtpClient::getUsageInfo(){}
static QoreValue FtpClient_getUsageInfo(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1238 "QC_FtpClient.qpp"
    return f->getUsageInfo();
}

// *string FtpClient::getUserName(){}
static QoreValue FtpClient_getUserName(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 730 "QC_FtpClient.qpp"
    const char *u = f->getUserName();
    return u ? new QoreStringNode(u) : nullptr;
}

// bool FtpClient::isDataSecure(){}
static QoreValue FtpClient_isDataSecure(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 945 "QC_FtpClient.qpp"
    return f->isDataSecure();
}

// bool FtpClient::isSecure(){}
static QoreValue FtpClient_isSecure(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 931 "QC_FtpClient.qpp"
    return f->isSecure();
}

// *string FtpClient::list(){}
static QoreValue FtpClient_list(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 213 "QC_FtpClient.qpp"
    return f->list(0, true, xsink);
}

// *string FtpClient::list(string path){}
static QoreValue FtpClient_list_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 238 "QC_FtpClient.qpp"
    return f->list(path->c_str(), true, xsink);
}

// nothing FtpClient::mkdir(string remote_path){}
static QoreValue FtpClient_mkdir_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 582 "QC_FtpClient.qpp"
    f->mkdir(remote_path->c_str(), xsink);
    return QoreValue();
}

// *string FtpClient::nlst(){}
static QoreValue FtpClient_nlst(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 260 "QC_FtpClient.qpp"
    return f->list(0, false, xsink);
}

// *string FtpClient::nlst(string path){}
static QoreValue FtpClient_nlst_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 284 "QC_FtpClient.qpp"
    return f->list(path->c_str(), false, xsink);
}

// nothing FtpClient::put(string local_path, *string remote_path){}
static QoreValue FtpClient_put_VsNs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* local_path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* remote_path = get_param_value(args, 1).get<const QoreStringNode>();
# 347 "QC_FtpClient.qpp"
    f->put(local_path->c_str(), remote_path ? remote_path->c_str() : 0, xsink);
    return QoreValue();
}

// nothing FtpClient::put(Qore::InputStream is, string remote_path){}
static QoreValue FtpClient_put_C11InputStreamVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "FtpClient::put()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 1);
# 372 "QC_FtpClient.qpp"
    SimpleRefHolder<InputStream> isHolder(is);
    f->put(is, remote_path->c_str(), xsink);
    return QoreValue();
}

// nothing FtpClient::putData(string data, string remote_path){}
static QoreValue FtpClient_putData_VsVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* data = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 1);
# 467 "QC_FtpClient.qpp"
    f->putData(data->c_str(), data->size(), remote_path->c_str(), xsink);
    return QoreValue();
}

// nothing FtpClient::putData(binary data, string remote_path){}
static QoreValue FtpClient_putData_VyVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 1);
# 488 "QC_FtpClient.qpp"
    f->putData(data->getPtr(), data->size(), remote_path->c_str(), xsink);
    return QoreValue();
}

// string FtpClient::pwd(){}
static QoreValue FtpClient_pwd(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 304 "QC_FtpClient.qpp"
    return f->pwd(xsink);
}

// nothing FtpClient::rename(string from, string to){}
static QoreValue FtpClient_rename_VsVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* from = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* to = HARD_QORE_VALUE_STRING(args, 1);
# 394 "QC_FtpClient.qpp"
    f->rename(from->c_str(), to->c_str(), xsink);
    return QoreValue();
}

// nothing FtpClient::rmdir(string remote_path){}
static QoreValue FtpClient_rmdir_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 602 "QC_FtpClient.qpp"
    f->rmdir(remote_path->c_str(), xsink);
    return QoreValue();
}

// hash<FtpResponseInfo> FtpClient::sendControlMessage(string cmd, string arg){}
static QoreValue FtpClient_sendControlMessage_VsVs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cmd = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* arg = HARD_QORE_VALUE_STRING(args, 1);
# 1284 "QC_FtpClient.qpp"
    return f->sendControlMessage(cmd->c_str(), arg->c_str(), xsink);
}

// nothing FtpClient::setControlEventQueue(){}
static QoreValue FtpClient_setControlEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1130 "QC_FtpClient.qpp"
    f->setControlEventQueue(xsink, nullptr, QoreValue(), false);
    return QoreValue();
}

// nothing FtpClient::setControlEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
static QoreValue FtpClient_setControlEventQueue_C5QueueVANb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setControlEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
    bool with_data = HARD_QORE_VALUE_BOOL(args, 2);
# 1150 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (queue->getMax() != -1) {
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which "
            "could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum "
            "size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
    }
    // pass reference from QoreObject::getReferencedPrivateData() to function
    f->setControlEventQueue(xsink, q.release(), arg.refSelf(), with_data);
    return QoreValue();
}

// nothing FtpClient::setDataEventQueue(){}
static QoreValue FtpClient_setDataEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1092 "QC_FtpClient.qpp"
    f->setDataEventQueue(xsink, nullptr, QoreValue(), false);
    return QoreValue();
}

// nothing FtpClient::setDataEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
static QoreValue FtpClient_setDataEventQueue_C5QueueVANb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setDataEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
    bool with_data = HARD_QORE_VALUE_BOOL(args, 2);
# 1112 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (queue->getMax() != -1) {
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which "
            "could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum "
            "size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
    }
    // pass reference from QoreObject::getReferencedPrivateData() to function
    f->setDataEventQueue(xsink, q.release(), arg.refSelf(), with_data);
    return QoreValue();
}

// nothing FtpClient::setEventQueue(){}
static QoreValue FtpClient_setEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1054 "QC_FtpClient.qpp"
    f->setEventQueue(xsink, nullptr, QoreValue(), false);
    return QoreValue();
}

// nothing FtpClient::setEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
static QoreValue FtpClient_setEventQueue_C5QueueVANb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
    bool with_data = HARD_QORE_VALUE_BOOL(args, 2);
# 1074 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (queue->getMax() != -1) {
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which "
            "could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum "
            "size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
    }
    // pass reference from QoreObject::getReferencedPrivateData() to function
    f->setEventQueue(xsink, q.release(), arg.refSelf(), with_data);
    return QoreValue();
}

// nothing FtpClient::setHostName(string host){}
static QoreValue FtpClient_setHostName_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
# 639 "QC_FtpClient.qpp"
    f->setHostName(host->c_str());
    return QoreValue();
}

// nothing FtpClient::setInsecure(){}
static QoreValue FtpClient_setInsecure(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 899 "QC_FtpClient.qpp"
    if (f->setInsecure()) {
        xsink->raiseException("SET-INSECURE-ERROR", "this method cannot be called while the control connection is "
            "established");
    }
    return QoreValue();
}

// nothing FtpClient::setInsecureData(){}
static QoreValue FtpClient_setInsecureData(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 916 "QC_FtpClient.qpp"
    if (f->setInsecureData()) {
        xsink->raiseException("SET-INSECUREDATA-ERROR", "this method cannot be called while the control connection is "
            "established");
    }
    return QoreValue();
}

// nothing FtpClient::setModeAuto(){}
static QoreValue FtpClient_setModeAuto(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 998 "QC_FtpClient.qpp"
    f->setModeAuto();
    return QoreValue();
}

// nothing FtpClient::setModeEPSV(){}
static QoreValue FtpClient_setModeEPSV(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1009 "QC_FtpClient.qpp"
    f->setModeEPSV();
    return QoreValue();
}

// nothing FtpClient::setModePASV(){}
static QoreValue FtpClient_setModePASV(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1020 "QC_FtpClient.qpp"
    f->setModePASV();
    return QoreValue();
}

// nothing FtpClient::setModePORT(){}
static QoreValue FtpClient_setModePORT(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1031 "QC_FtpClient.qpp"
    f->setModePORT();
    return QoreValue();
}

// nothing FtpClient::setNetworkFamily(int family){}
static QoreValue FtpClient_setNetworkFamily_Vi(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 family = HARD_QORE_VALUE_INT(args, 0);
# 699 "QC_FtpClient.qpp"
    if (family != AF_INET && family != AF_INET6 && family != AF_UNSPEC) {
        xsink->raiseException("FTPCLIENT-SETNETWORKFAMILY-ERROR", "invalid network family " QLLD, family);
        return QoreValue();
    }
    f->setNetworkFamily(family);
    return QoreValue();
}

// nothing FtpClient::setPassword(string pass){}
static QoreValue FtpClient_setPassword_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 0);
# 626 "QC_FtpClient.qpp"
    f->setPassword(pass->c_str());
    return QoreValue();
}

// nothing FtpClient::setPort(int port){}
static QoreValue FtpClient_setPort_Vi(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 port = HARD_QORE_VALUE_INT(args, 0);
# 653 "QC_FtpClient.qpp"
    if (port <= 0) {
        xsink->raiseException("FTPCLIENT-SETPORT-PARAMETER-ERROR", "expecting positive port number as first "
            "parameter of FtpClient::setPort(softint port); got %d", port);
    } else {
        f->setPort(port);
    }
    return QoreValue();
}

// nothing FtpClient::setSecure(bool secure = True){}
static QoreValue FtpClient_setSecure_Vb(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool secure = HARD_QORE_VALUE_BOOL(args, 0);
# 883 "QC_FtpClient.qpp"
    if (secure ? f->setSecure() : f->setInsecure()) {
        xsink->raiseException("SET-SECURE-ERROR", "this method cannot be called while the control connection is "
            "established");
    }
    return QoreValue();
}

// nothing FtpClient::setTimeout(timeout timeout_ms){}
static QoreValue FtpClient_setTimeout_Vt(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1270 "QC_FtpClient.qpp"
    f->setTimeout(timeout_ms);
    return QoreValue();
}

// nothing FtpClient::setURL(string url){}
static QoreValue FtpClient_setURL_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 675 "QC_FtpClient.qpp"
    f->setURL(url, xsink);

    const char* url_path = f->getUrlPath();
    if (url_path) {
        ValueHolder val(new QoreStringNode(url_path), xsink);
        if (self->setMemberValue("path", QC_FTPCLIENT, *val, xsink)) {
            return QoreValue();
        }
    }
    return QoreValue();
}

// nothing FtpClient::setUserName(string user){}
static QoreValue FtpClient_setUserName_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
# 614 "QC_FtpClient.qpp"
    f->setUserName(user->c_str());
    return QoreValue();
}

// nothing FtpClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
static QoreValue FtpClient_setWarningQueue_ViViC5QueueVAVt(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_ms = HARD_QORE_VALUE_INT(args, 0);
    int64 warning_bs = HARD_QORE_VALUE_INT(args, 1);
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "FtpClient::setWarningQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 3);
    int64 min_ms = HARD_QORE_VALUE_INT(args, 4);
# 1208 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
    if (queue->getMax() != -1) {
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which "
            "could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum "
            "size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
    }
    // pass reference from QoreObject::getReferencedPrivateData() to function
    f->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg.refSelf(), min_ms);
    return QoreValue();
}

// *string FtpClient::verifyPeerCertificate(){}
static QoreValue FtpClient_verifyPeerCertificate(QoreObject* self, QoreFtpClientClass* f, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 986 "QC_FtpClient.qpp"
    const char *c = getSSLCVCode(f->verifyPeerCertificate());
    return c ? new QoreStringNode(c) : nullptr;
}

DLLLOCAL void preinitFtpClientClass() {
    QC_FTPCLIENT = new QoreClass("FtpClient", "::Qore::FtpClient", QDOM_NETWORK);
    CID_FTPCLIENT = QC_FTPCLIENT->getID();
    QC_FTPCLIENT->setSystem();
}

DLLLOCAL QoreClass* initFtpClientClass(QoreNamespace& ns) {
    if (!QC_FTPCLIENT)
        preinitFtpClientClass();

    // public members
    QC_FTPCLIENT->addMember("path", Public, stringTypeInfo);

    // nothing FtpClient::clearStats(){}
    QC_FTPCLIENT->addMethod("clearStats", (q_method_n_t)FtpClient_clearStats, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::clearWarningQueue(){}
    QC_FTPCLIENT->addMethod("clearWarningQueue", (q_method_n_t)FtpClient_clearWarningQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::connect(){}
    QC_FTPCLIENT->addMethod("connect", (q_method_n_t)FtpClient_connect, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // FtpClient::constructor() {}
    QC_FTPCLIENT->addConstructor(FtpClient_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // FtpClient::constructor(string url) {}
    QC_FTPCLIENT->addConstructor(FtpClient_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "url");

    // FtpClient::copy() {}
    QC_FTPCLIENT->setCopy((q_copy_t)FtpClient_copy);

    // nothing FtpClient::cwd(string path){}
    QC_FTPCLIENT->addMethod("cwd", (q_method_n_t)FtpClient_cwd_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // nothing FtpClient::del(string remote_path){}
    QC_FTPCLIENT->addMethod("del", (q_method_n_t)FtpClient_del_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // FtpClient::destructor() {}
    QC_FTPCLIENT->setDestructor((q_destructor_t)FtpClient_destructor);

    // nothing FtpClient::disconnect(){}
    QC_FTPCLIENT->addMethod("disconnect", (q_method_n_t)FtpClient_disconnect, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::get(string remote_path, *string local_path){}
    QC_FTPCLIENT->addMethod("get", (q_method_n_t)FtpClient_get_VsNs, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "local_path");

    // nothing FtpClient::get(string remote_path, Qore::OutputStream os){}
    QC_FTPCLIENT->addMethod("get", (q_method_n_t)FtpClient_get_VsC12OutputStream, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path", QC_OUTPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "os");

    // binary FtpClient::getAsBinary(string remote_path){}
    QC_FTPCLIENT->addMethod("getAsBinary", (q_method_n_t)FtpClient_getAsBinary_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // string FtpClient::getAsString(string remote_path, *string encoding){}
    QC_FTPCLIENT->addMethod("getAsString", (q_method_n_t)FtpClient_getAsString_VsNs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // hash<auto> FtpClient::getControlPeerInfo(bool host_lookup = True){}
    QC_FTPCLIENT->addMethod("getControlPeerInfo", (q_method_n_t)FtpClient_getControlPeerInfo_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "host_lookup");

    // hash<auto> FtpClient::getControlSocketInfo(bool host_lookup = True){}
    QC_FTPCLIENT->addMethod("getControlSocketInfo", (q_method_n_t)FtpClient_getControlSocketInfo_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "host_lookup");

    // hash<auto> FtpClient::getDataPeerInfo(bool host_lookup = True){}
    QC_FTPCLIENT->addMethod("getDataPeerInfo", (q_method_n_t)FtpClient_getDataPeerInfo_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "host_lookup");

    // hash<auto> FtpClient::getDataSocketInfo(bool host_lookup = True){}
    QC_FTPCLIENT->addMethod("getDataSocketInfo", (q_method_n_t)FtpClient_getDataSocketInfo_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "host_lookup");

    // *string FtpClient::getHostName(){}
    QC_FTPCLIENT->addMethod("getHostName", (q_method_n_t)FtpClient_getHostName, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string FtpClient::getMode(){}
    QC_FTPCLIENT->addMethod("getMode", (q_method_n_t)FtpClient_getMode, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int FtpClient::getNetworkFamily(){}
    QC_FTPCLIENT->addMethod("getNetworkFamily", (q_method_n_t)FtpClient_getNetworkFamily, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string FtpClient::getPassword(){}
    QC_FTPCLIENT->addMethod("getPassword", (q_method_n_t)FtpClient_getPassword, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int FtpClient::getPort(){}
    QC_FTPCLIENT->addMethod("getPort", (q_method_n_t)FtpClient_getPort, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *string FtpClient::getSSLCipherName(){}
    QC_FTPCLIENT->addMethod("getSSLCipherName", (q_method_n_t)FtpClient_getSSLCipherName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string FtpClient::getSSLCipherVersion(){}
    QC_FTPCLIENT->addMethod("getSSLCipherVersion", (q_method_n_t)FtpClient_getSSLCipherVersion, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int FtpClient::getTimeout(){}
    QC_FTPCLIENT->addMethod("getTimeout", (q_method_n_t)FtpClient_getTimeout, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string FtpClient::getURL(){}
    QC_FTPCLIENT->addMethod("getURL", (q_method_n_t)FtpClient_getURL, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // hash FtpClient::getUsageInfo(){}
    QC_FTPCLIENT->addMethod("getUsageInfo", (q_method_n_t)FtpClient_getUsageInfo, Public, QCF_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // *string FtpClient::getUserName(){}
    QC_FTPCLIENT->addMethod("getUserName", (q_method_n_t)FtpClient_getUserName, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // bool FtpClient::isDataSecure(){}
    QC_FTPCLIENT->addMethod("isDataSecure", (q_method_n_t)FtpClient_isDataSecure, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool FtpClient::isSecure(){}
    QC_FTPCLIENT->addMethod("isSecure", (q_method_n_t)FtpClient_isSecure, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string FtpClient::list(){}
    QC_FTPCLIENT->addMethod("list", (q_method_n_t)FtpClient_list, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string FtpClient::list(string path){}
    QC_FTPCLIENT->addMethod("list", (q_method_n_t)FtpClient_list_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // nothing FtpClient::mkdir(string remote_path){}
    QC_FTPCLIENT->addMethod("mkdir", (q_method_n_t)FtpClient_mkdir_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // *string FtpClient::nlst(){}
    QC_FTPCLIENT->addMethod("nlst", (q_method_n_t)FtpClient_nlst, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string FtpClient::nlst(string path){}
    QC_FTPCLIENT->addMethod("nlst", (q_method_n_t)FtpClient_nlst_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "path");

    // nothing FtpClient::put(string local_path, *string remote_path){}
    QC_FTPCLIENT->addMethod("put", (q_method_n_t)FtpClient_put_VsNs, Public, QCF_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "local_path", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // nothing FtpClient::put(Qore::InputStream is, string remote_path){}
    QC_FTPCLIENT->addMethod("put", (q_method_n_t)FtpClient_put_C11InputStreamVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_INPUTSTREAM->getTypeInfo(), QORE_PARAM_NO_ARG, "is", stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // nothing FtpClient::putData(string data, string remote_path){}
    QC_FTPCLIENT->addMethod("putData", (q_method_n_t)FtpClient_putData_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "data", stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // nothing FtpClient::putData(binary data, string remote_path){}
    QC_FTPCLIENT->addMethod("putData", (q_method_n_t)FtpClient_putData_VyVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // string FtpClient::pwd(){}
    QC_FTPCLIENT->addMethod("pwd", (q_method_n_t)FtpClient_pwd, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // nothing FtpClient::rename(string from, string to){}
    QC_FTPCLIENT->addMethod("rename", (q_method_n_t)FtpClient_rename_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "from", stringTypeInfo, QORE_PARAM_NO_ARG, "to");

    // nothing FtpClient::rmdir(string remote_path){}
    QC_FTPCLIENT->addMethod("rmdir", (q_method_n_t)FtpClient_rmdir_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "remote_path");

    // hash<FtpResponseInfo> FtpClient::sendControlMessage(string cmd, string arg){}
    QC_FTPCLIENT->addMethod("sendControlMessage", (q_method_n_t)FtpClient_sendControlMessage_VsVs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, hashdeclFtpResponseInfo->getTypeInfo(false), 2, stringTypeInfo, QORE_PARAM_NO_ARG, "cmd", stringTypeInfo, QORE_PARAM_NO_ARG, "arg");

    // nothing FtpClient::setControlEventQueue(){}
    QC_FTPCLIENT->addMethod("setControlEventQueue", (q_method_n_t)FtpClient_setControlEventQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setControlEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
    QC_FTPCLIENT->addMethod("setControlEventQueue", (q_method_n_t)FtpClient_setControlEventQueue_C5QueueVANb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "with_data");

    // nothing FtpClient::setDataEventQueue(){}
    QC_FTPCLIENT->addMethod("setDataEventQueue", (q_method_n_t)FtpClient_setDataEventQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setDataEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
    QC_FTPCLIENT->addMethod("setDataEventQueue", (q_method_n_t)FtpClient_setDataEventQueue_C5QueueVANb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "with_data");

    // nothing FtpClient::setEventQueue(){}
    QC_FTPCLIENT->addMethod("setEventQueue", (q_method_n_t)FtpClient_setEventQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setEventQueue(Qore::Thread::Queue queue, auto arg, *bool with_data){}
    QC_FTPCLIENT->addMethod("setEventQueue", (q_method_n_t)FtpClient_setEventQueue_C5QueueVANb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 3, QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", boolOrNothingTypeInfo, QORE_PARAM_NO_ARG, "with_data");

    // nothing FtpClient::setHostName(string host){}
    QC_FTPCLIENT->addMethod("setHostName", (q_method_n_t)FtpClient_setHostName_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "host");

    // nothing FtpClient::setInsecure(){}
    QC_FTPCLIENT->addMethod("setInsecure", (q_method_n_t)FtpClient_setInsecure, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setInsecureData(){}
    QC_FTPCLIENT->addMethod("setInsecureData", (q_method_n_t)FtpClient_setInsecureData, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModeAuto(){}
    QC_FTPCLIENT->addMethod("setModeAuto", (q_method_n_t)FtpClient_setModeAuto, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModeEPSV(){}
    QC_FTPCLIENT->addMethod("setModeEPSV", (q_method_n_t)FtpClient_setModeEPSV, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModePASV(){}
    QC_FTPCLIENT->addMethod("setModePASV", (q_method_n_t)FtpClient_setModePASV, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModePORT(){}
    QC_FTPCLIENT->addMethod("setModePORT", (q_method_n_t)FtpClient_setModePORT, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setNetworkFamily(int family){}
    QC_FTPCLIENT->addMethod("setNetworkFamily", (q_method_n_t)FtpClient_setNetworkFamily_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "family");

    // nothing FtpClient::setPassword(string pass){}
    QC_FTPCLIENT->addMethod("setPassword", (q_method_n_t)FtpClient_setPassword_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "pass");

    // nothing FtpClient::setPort(int port){}
    QC_FTPCLIENT->addMethod("setPort", (q_method_n_t)FtpClient_setPort_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "port");

    // nothing FtpClient::setSecure(bool secure = True){}
    QC_FTPCLIENT->addMethod("setSecure", (q_method_n_t)FtpClient_setSecure_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "secure");

    // nothing FtpClient::setTimeout(timeout timeout_ms){}
    QC_FTPCLIENT->addMethod("setTimeout", (q_method_n_t)FtpClient_setTimeout_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, QORE_PARAM_NO_ARG, "timeout_ms");

    // nothing FtpClient::setURL(string url){}
    QC_FTPCLIENT->addMethod("setURL", (q_method_n_t)FtpClient_setURL_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "url");

    // nothing FtpClient::setUserName(string user){}
    QC_FTPCLIENT->addMethod("setUserName", (q_method_n_t)FtpClient_setUserName_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "user");

    // nothing FtpClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
    QC_FTPCLIENT->addMethod("setWarningQueue", (q_method_n_t)FtpClient_setWarningQueue_ViViC5QueueVAVt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, QORE_PARAM_NO_ARG, "warning_ms", bigIntTypeInfo, QORE_PARAM_NO_ARG, "warning_bs", QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg", timeoutTypeInfo, QoreSimpleValue().assign(DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0)), "min_ms");

    // *string FtpClient::verifyPeerCertificate(){}
    QC_FTPCLIENT->addMethod("verifyPeerCertificate", (q_method_n_t)FtpClient_verifyPeerCertificate, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    return QC_FTPCLIENT;
}
