/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashListIterator.qpp HashListIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreHashListIterator.h"

extern QoreClass* QC_HASHLISTITERATOR;

// internal reverse iterator class implementation only for the getName() function - the iterators remain
// forwards and are used in the reverse sense by the Qore language class implementation below
class QoreHashListReverseIterator : public QoreHashListIterator {
public:
    DLLLOCAL QoreHashListReverseIterator(const QoreHashNode* n_h) : QoreHashListIterator(n_h) {
    }

    DLLLOCAL QoreHashListReverseIterator() {
    }

    DLLLOCAL QoreHashListReverseIterator(const QoreHashListReverseIterator& old) : QoreHashListIterator(old) {
    }

    DLLLOCAL virtual const char* getName() const {
        return "HashListReverseIterator";
    }
};
/* Qore class Qore::HashListReverseIterator */

qore_classid_t CID_HASHLISTREVERSEITERATOR;
QoreClass* QC_HASHLISTREVERSEITERATOR;

// HashListReverseIterator::constructor(hash<auto> h) {}
static void HashListReverseIterator_constructor_C10hash_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 106 "QC_HashListReverseIterator.qpp"
self->setPrivate(CID_HASHLISTREVERSEITERATOR, new QoreHashListReverseIterator(h));
}

// HashListReverseIterator::constructor() {}
static void HashListReverseIterator_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 117 "QC_HashListReverseIterator.qpp"
self->setPrivate(CID_HASHLISTREVERSEITERATOR, new QoreHashListReverseIterator);
}

// HashListReverseIterator::copy() {}
static void HashListReverseIterator_copy(QoreObject* self, QoreObject* old, QoreHashListReverseIterator* i, ExceptionSink* xsink) {
# 127 "QC_HashListReverseIterator.qpp"
self->setPrivate(CID_HASHLISTREVERSEITERATOR, new QoreHashListReverseIterator(*i));
}

// bool HashListReverseIterator::first(){}
static QoreValue HashListReverseIterator_first(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 184 "QC_HashListReverseIterator.qpp"
    return i->last();
}

// bool HashListReverseIterator::last(){}
static QoreValue HashListReverseIterator_last(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 199 "QC_HashListReverseIterator.qpp"
    return i->first();
}

// auto HashListReverseIterator::memberGate(string key){}
static QoreValue HashListReverseIterator_memberGate_Vs(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 221 "QC_HashListReverseIterator.qpp"
    if (i->check(xsink))
        return QoreValue();

    return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool HashListReverseIterator::next(){}
static QoreValue HashListReverseIterator_next(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 146 "QC_HashListReverseIterator.qpp"
    if (i->check(xsink))
        return false;
    return i->prev();
}

// bool HashListReverseIterator::prev(){}
static QoreValue HashListReverseIterator_prev(QoreObject* self, QoreHashListReverseIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 167 "QC_HashListReverseIterator.qpp"
    if (i->check(xsink))
        return false;
    return i->next();
}

DLLLOCAL void preinitHashListReverseIteratorClass() {
    QC_HASHLISTREVERSEITERATOR = new QoreClass("HashListReverseIterator", "::Qore::HashListReverseIterator", QDOM_DEFAULT);
    CID_HASHLISTREVERSEITERATOR = QC_HASHLISTREVERSEITERATOR->getID();
    QC_HASHLISTREVERSEITERATOR->setSystem();
}

DLLLOCAL QoreClass* initHashListReverseIteratorClass(QoreNamespace& ns) {
    if (!QC_HASHLISTREVERSEITERATOR)
        preinitHashListReverseIteratorClass();

    // set parent class
    assert(QC_HASHLISTITERATOR);
    QC_HASHLISTREVERSEITERATOR->addBuiltinVirtualBaseClass(QC_HASHLISTITERATOR);

    // HashListReverseIterator::constructor(hash<auto> h) {}
    QC_HASHLISTREVERSEITERATOR->addConstructor(HashListReverseIterator_constructor_C10hash_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, autoHashTypeInfo, QORE_PARAM_NO_ARG, "h");

    // HashListReverseIterator::constructor() {}
    QC_HASHLISTREVERSEITERATOR->addConstructor(HashListReverseIterator_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // HashListReverseIterator::copy() {}
    QC_HASHLISTREVERSEITERATOR->setCopy((q_copy_t)HashListReverseIterator_copy);

    // bool HashListReverseIterator::first(){}
    QC_HASHLISTREVERSEITERATOR->addMethod("first", (q_method_n_t)HashListReverseIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HashListReverseIterator::last(){}
    QC_HASHLISTREVERSEITERATOR->addMethod("last", (q_method_n_t)HashListReverseIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto HashListReverseIterator::memberGate(string key){}
    QC_HASHLISTREVERSEITERATOR->addMethod("memberGate", (q_method_n_t)HashListReverseIterator_memberGate_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "key");

    // bool HashListReverseIterator::next(){}
    QC_HASHLISTREVERSEITERATOR->addMethod("next", (q_method_n_t)HashListReverseIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool HashListReverseIterator::prev(){}
    QC_HASHLISTREVERSEITERATOR->addMethod("prev", (q_method_n_t)HashListReverseIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    return QC_HASHLISTREVERSEITERATOR;
}
