/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListIterator.qpp ListIterator class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreListIterator.h"
/* Qore class Qore::ListIterator */

qore_classid_t CID_LISTITERATOR;
QoreClass* QC_LISTITERATOR;

// ListIterator::constructor(softlist<auto> l) {}
static void ListIterator_constructor_C14softlist_auto_(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 72 "QC_ListIterator.qpp"
self->setPrivate(CID_LISTITERATOR, new QoreListIterator(l));
}

// ListIterator::copy() {}
static void ListIterator_copy(QoreObject* self, QoreObject* old, QoreListIterator* i, ExceptionSink* xsink) {
# 82 "QC_ListIterator.qpp"
self->setPrivate(CID_LISTITERATOR, new QoreListIterator(*i));
}

// bool ListIterator::empty(){}
static QoreValue ListIterator_empty(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 137 "QC_ListIterator.qpp"
    return i->empty();
}

// bool ListIterator::first(){}
static QoreValue ListIterator_first(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 152 "QC_ListIterator.qpp"
    return i->first();
}

// auto ListIterator::getValue(){}
static QoreValue ListIterator_getValue(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 184 "QC_ListIterator.qpp"
    if (i->check(xsink))
        return QoreValue();
    return i->getReferencedValue(xsink);
}

// int ListIterator::index(){}
static QoreValue ListIterator_index(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 200 "QC_ListIterator.qpp"
    return i->index();
}

// bool ListIterator::last(){}
static QoreValue ListIterator_last(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 167 "QC_ListIterator.qpp"
    return i->last();
}

// int ListIterator::max(){}
static QoreValue ListIterator_max(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 214 "QC_ListIterator.qpp"
    return i->max();
}

// bool ListIterator::next(){}
static QoreValue ListIterator_next(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 101 "QC_ListIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next();
}

// bool ListIterator::prev(){}
static QoreValue ListIterator_prev(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 122 "QC_ListIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing ListIterator::reset(){}
static QoreValue ListIterator_reset(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 260 "QC_ListIterator.qpp"
    if (i->check(xsink))
       return QoreValue();
   i->reset();
    return QoreValue();
}

// bool ListIterator::set(int pos){}
static QoreValue ListIterator_set_Vi(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 pos = HARD_QORE_VALUE_INT(args, 0);
# 231 "QC_ListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return !i->set(pos);
}

// bool ListIterator::valid(){}
static QoreValue ListIterator_valid(QoreObject* self, QoreListIterator* i, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 246 "QC_ListIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitListIteratorClass() {
    QC_LISTITERATOR = new QoreClass("ListIterator", "::Qore::ListIterator", QDOM_DEFAULT);
    CID_LISTITERATOR = QC_LISTITERATOR->getID();
    QC_LISTITERATOR->setSystem();
}

DLLLOCAL QoreClass* initListIteratorClass(QoreNamespace& ns) {
    if (!QC_LISTITERATOR)
        preinitListIteratorClass();

    // set parent class
    assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
    QC_LISTITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

    // ListIterator::constructor(softlist<auto> l) {}
    QC_LISTITERATOR->addConstructor(ListIterator_constructor_C14softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, softAutoListTypeInfo, QORE_PARAM_NO_ARG, "l");

    // ListIterator::copy() {}
    QC_LISTITERATOR->setCopy((q_copy_t)ListIterator_copy);

    // bool ListIterator::empty(){}
    QC_LISTITERATOR->addMethod("empty", (q_method_n_t)ListIterator_empty, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ListIterator::first(){}
    QC_LISTITERATOR->addMethod("first", (q_method_n_t)ListIterator_first, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto ListIterator::getValue(){}
    QC_LISTITERATOR->addMethod("getValue", (q_method_n_t)ListIterator_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // int ListIterator::index(){}
    QC_LISTITERATOR->addMethod("index", (q_method_n_t)ListIterator_index, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool ListIterator::last(){}
    QC_LISTITERATOR->addMethod("last", (q_method_n_t)ListIterator_last, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int ListIterator::max(){}
    QC_LISTITERATOR->addMethod("max", (q_method_n_t)ListIterator_max, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool ListIterator::next(){}
    QC_LISTITERATOR->addMethod("next", (q_method_n_t)ListIterator_next, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool ListIterator::prev(){}
    QC_LISTITERATOR->addMethod("prev", (q_method_n_t)ListIterator_prev, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing ListIterator::reset(){}
    QC_LISTITERATOR->addMethod("reset", (q_method_n_t)ListIterator_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool ListIterator::set(int pos){}
    QC_LISTITERATOR->addMethod("set", (q_method_n_t)ListIterator_set_Vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "pos");

    // bool ListIterator::valid(){}
    QC_LISTITERATOR->addMethod("valid", (q_method_n_t)ListIterator_valid, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_LISTITERATOR;
}
