/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StreamBase.qpp StreamBase class (interface) definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/StreamBase.h"
/* Qore class Qore::StreamBase */

qore_classid_t CID_STREAMBASE;
QoreClass* QC_STREAMBASE;

// StreamBase::constructor() {}
static void StreamBase_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 53 "QC_StreamBase.qpp"
xsink->raiseException("STREAMBASE-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

// int StreamBase::getThreadId(){}
static QoreValue StreamBase_getThreadId(QoreObject* self, StreamBase* sb, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 84 "QC_StreamBase.qpp"
    return sb->getThreadId();
}

// nothing StreamBase::reassignThread(){}
static QoreValue StreamBase_reassignThread(QoreObject* self, StreamBase* sb, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 65 "QC_StreamBase.qpp"
    	sb->reassignThread(xsink);
    return QoreValue();
}

// nothing StreamBase::unassignThread(){}
static QoreValue StreamBase_unassignThread(QoreObject* self, StreamBase* sb, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 77 "QC_StreamBase.qpp"
    	sb->unassignThread(xsink);
    return QoreValue();
}

DLLLOCAL void preinitStreamBaseClass() {
    QC_STREAMBASE = new QoreClass("StreamBase", "::Qore::StreamBase", QDOM_DEFAULT);
    CID_STREAMBASE = QC_STREAMBASE->getID();
    QC_STREAMBASE->setSystem();
}

DLLLOCAL QoreClass* initStreamBaseClass(QoreNamespace& ns) {
    if (!QC_STREAMBASE)
        preinitStreamBaseClass();

    // StreamBase::constructor() {}
    QC_STREAMBASE->addConstructor(StreamBase_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // int StreamBase::getThreadId(){}
    QC_STREAMBASE->addMethod("getThreadId", (q_method_n_t)StreamBase_getThreadId, Public, QCF_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing StreamBase::reassignThread(){}
    QC_STREAMBASE->addMethod("reassignThread", (q_method_n_t)StreamBase_reassignThread, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing StreamBase::unassignThread(){}
    QC_STREAMBASE->addMethod("unassignThread", (q_method_n_t)StreamBase_unassignThread, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    return QC_STREAMBASE;
}
