/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StreamPipe.qpp StreamPipe class definition */
/*
    Qore Programming Language

    Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore/Qore.h"
#include "qore/intern/StreamPipe.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/QoreClassIntern.h"

DLLEXPORT extern QoreClass* QC_PIPEINPUTSTREAM;
DLLEXPORT extern QoreClass* QC_PIPEOUTPUTSTREAM;
/* Qore class Qore::StreamPipe */

qore_classid_t CID_STREAMPIPE;
QoreClass* QC_STREAMPIPE;

// StreamPipe::constructor(bool syncClose = True, timeout timeout_ms = -1, int bufferSize = 4096) {}
static void StreamPipe_constructor_VbVtVi(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool syncClose = HARD_QORE_VALUE_BOOL(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
    int64 bufferSize = HARD_QORE_VALUE_INT(args, 2);
# 116 "QC_StreamPipe.qpp"
SimpleRefHolder<StreamPipe> pipe(new StreamPipe(syncClose, timeout_ms, bufferSize, xsink));
    qore_object_private* o = qore_object_private::get(*self);
    const qore_class_private* cls = qore_class_private::get(*QC_STREAMPIPE);
    pipe->ref();
    o->setValueIntern(cls, "is", new QoreObject(QC_PIPEINPUTSTREAM, getProgram(), new PipeInputStream(*pipe)), xsink);
    pipe->ref();
    o->setValueIntern(cls, "os", new QoreObject(QC_PIPEOUTPUTSTREAM, getProgram(), new PipeOutputStream(*pipe)), xsink);
    self->setPrivate(CID_STREAMPIPE, pipe.release());
}

// PipeInputStream StreamPipe::getInputStream(){}
static QoreValue StreamPipe_getInputStream(QoreObject* self, StreamPipe* pipe, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 131 "QC_StreamPipe.qpp"
    return self->getReferencedMemberNoMethod("is", xsink);
}

// PipeOutputStream StreamPipe::getOutputStream(){}
static QoreValue StreamPipe_getOutputStream(QoreObject* self, StreamPipe* pipe, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 139 "QC_StreamPipe.qpp"
    return self->getReferencedMemberNoMethod("os", xsink);
}

DLLLOCAL void preinitStreamPipeClass() {
    QC_STREAMPIPE = new QoreClass("StreamPipe", "::Qore::StreamPipe", QDOM_DEFAULT);
    CID_STREAMPIPE = QC_STREAMPIPE->getID();
    QC_STREAMPIPE->setSystem();
}

DLLLOCAL QoreClass* initStreamPipeClass(QoreNamespace& ns) {
    if (!QC_STREAMPIPE)
        preinitStreamPipeClass();

    // private:internal members
    QC_STREAMPIPE->addMember("os", Internal, QC_OUTPUTSTREAM->getTypeInfo());
    QC_STREAMPIPE->addMember("is", Internal, QC_INPUTSTREAM->getTypeInfo());

    QC_STREAMPIPE->setFinal();

    // StreamPipe::constructor(bool syncClose = True, timeout timeout_ms = -1, int bufferSize = 4096) {}
    QC_STREAMPIPE->addConstructor(StreamPipe_constructor_VbVtVi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, boolTypeInfo, QoreSimpleValue().assign(true), "syncClose", timeoutTypeInfo, QoreSimpleValue().assign((int64)-1), "timeout_ms", bigIntTypeInfo, QoreSimpleValue().assign((int64)4096), "bufferSize");

    // PipeInputStream StreamPipe::getInputStream(){}
    QC_STREAMPIPE->addMethod("getInputStream", (q_method_n_t)StreamPipe_getInputStream, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_PIPEINPUTSTREAM->getTypeInfo());

    // PipeOutputStream StreamPipe::getOutputStream(){}
    QC_STREAMPIPE->addMethod("getOutputStream", (q_method_n_t)StreamPipe_getOutputStream, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_PIPEOUTPUTSTREAM->getTypeInfo());

    return QC_STREAMPIPE;
}
