/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_StringOutputStream.qpp StringOutputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/StringOutputStream.h"
/* Qore class Qore::StringOutputStream */

qore_classid_t CID_STRINGOUTPUTSTREAM;
QoreClass* QC_STRINGOUTPUTSTREAM;

// nothing StringOutputStream::close(){}
static QoreValue StringOutputStream_close(QoreObject* self, StringOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 91 "QC_StringOutputStream.qpp"
    os->closeHelper(xsink);
    return QoreValue();
}

// StringOutputStream::constructor() {}
static void StringOutputStream_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 67 "QC_StringOutputStream.qpp"
SimpleRefHolder<StringOutputStream> sos(new StringOutputStream);
    self->setPrivate(CID_STRINGOUTPUTSTREAM, sos.release());
}

// StringOutputStream::constructor(string encoding) {}
static void StringOutputStream_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 79 "QC_StringOutputStream.qpp"
SimpleRefHolder<StringOutputStream> sos(new StringOutputStream(QEM.findCreate(encoding)));
   self->setPrivate(CID_STRINGOUTPUTSTREAM, sos.release());
}

// string StringOutputStream::getData(){}
static QoreValue StringOutputStream_getData(QoreObject* self, StringOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 133 "QC_StringOutputStream.qpp"
    return os->getData(xsink);
}

// string StringOutputStream::getEncoding(){}
static QoreValue StringOutputStream_getEncoding(QoreObject* self, StringOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 168 "QC_StringOutputStream.qpp"
    return new QoreStringNode(os->getEncoding());
}

// bool StringOutputStream::hasData(){}
static QoreValue StringOutputStream_hasData(QoreObject* self, StringOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 151 "QC_StringOutputStream.qpp"
    return os->hasData(xsink);
}

// nothing StringOutputStream::write(binary data){}
static QoreValue StringOutputStream_write_Vy(QoreObject* self, StringOutputStream* os, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 112 "QC_StringOutputStream.qpp"
    os->writeHelper(data, xsink);
    return QoreValue();
}

DLLLOCAL void preinitStringOutputStreamClass() {
    QC_STRINGOUTPUTSTREAM = new QoreClass("StringOutputStream", "::Qore::StringOutputStream", QDOM_DEFAULT);
    CID_STRINGOUTPUTSTREAM = QC_STRINGOUTPUTSTREAM->getID();
    QC_STRINGOUTPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initStringOutputStreamClass(QoreNamespace& ns) {
    if (!QC_STRINGOUTPUTSTREAM)
        preinitStringOutputStreamClass();

    // set parent class
    assert(QC_OUTPUTSTREAM);
    QC_STRINGOUTPUTSTREAM->addBuiltinVirtualBaseClass(QC_OUTPUTSTREAM);

    QC_STRINGOUTPUTSTREAM->setFinal();

    // nothing StringOutputStream::close(){}
    QC_STRINGOUTPUTSTREAM->addMethod("close", (q_method_n_t)StringOutputStream_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // StringOutputStream::constructor() {}
    QC_STRINGOUTPUTSTREAM->addConstructor(StringOutputStream_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // StringOutputStream::constructor(string encoding) {}
    QC_STRINGOUTPUTSTREAM->addConstructor(StringOutputStream_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // string StringOutputStream::getData(){}
    QC_STRINGOUTPUTSTREAM->addMethod("getData", (q_method_n_t)StringOutputStream_getData, Public, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // string StringOutputStream::getEncoding(){}
    QC_STRINGOUTPUTSTREAM->addMethod("getEncoding", (q_method_n_t)StringOutputStream_getEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool StringOutputStream::hasData(){}
    QC_STRINGOUTPUTSTREAM->addMethod("hasData", (q_method_n_t)StringOutputStream_hasData, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // nothing StringOutputStream::write(binary data){}
    QC_STRINGOUTPUTSTREAM->addMethod("write", (q_method_n_t)StringOutputStream_write_Vy, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, QORE_PARAM_NO_ARG, "data");

    return QC_STRINGOUTPUTSTREAM;
}
