/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_context.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2023 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/Context.h"

static AbstractQoreNode* cx_err(const char* func, ExceptionSink* xsink) {
   return xsink->raiseException("CONTEXT-ERROR", "function %s() called without a current data iteration context", func);
}
# 40 "ql_context.qpp"
# 73 "ql_context.qpp"
# 97 "ql_context.qpp"
# 121 "ql_context.qpp"
# 144 "ql_context.qpp"
// bool cx_first() {}
static QoreValue f_cx_first(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 89 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c)
      return cx_err("cx_first", xsink);

   return c->isFirst();
}

// bool cx_last() {}
static QoreValue f_cx_last(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 113 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c)
      return cx_err("cx_last", xsink);

   return c->isLast();
}

// int cx_pos() {}
static QoreValue f_cx_pos(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 136 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c)
      return cx_err("cx_pos", xsink);

   return c->getPos();
}

// int cx_total() {}
static QoreValue f_cx_total(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 159 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c)
      return cx_err("cx_total", xsink);

   return c->getTotal();
}

// auto cx_value(string key) {}
static QoreValue f_cx_value_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 61 "ql_context.qpp"
   Context* c = get_context_stack();
   if (!c)
      return cx_err("cx_value", xsink);

   TempEncodingHelper tmp(key, QCS_DEFAULT, xsink);
   if (!tmp)
      return QoreValue();

   return c->eval(tmp->getBuffer(), xsink);
}


DLLLOCAL void init_context_functions(QoreNamespace& ns) {
    // bool cx_first() {}
    ns.addBuiltinVariant("cx_first", (q_func_n_t)f_cx_first, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);
    // bool cx_last() {}
    ns.addBuiltinVariant("cx_last", (q_func_n_t)f_cx_last, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);
    // int cx_pos() {}
    ns.addBuiltinVariant("cx_pos", (q_func_n_t)f_cx_pos, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);
    // int cx_total() {}
    ns.addBuiltinVariant("cx_total", (q_func_n_t)f_cx_total, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);
    // auto cx_value(string key) {}
    ns.addBuiltinVariant("cx_value", (q_func_n_t)f_cx_value_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "key");

}
