/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AstTree.cpp AstTree class definition */
/*
  Qore AST Parser

  Copyright (C) 2016 - 2023 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <fstream>

#include "qore/Qore.h"

#include "AstTreeHolder.h"
#include "ast/ASTParseError.h"
/* Qore class astparser::AstTree */

qore_classid_t CID_ASTTREE;
QoreClass* QC_ASTTREE;

// AstTree::constructor() {}
static void AstTree_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 45 "QC_AstTree.qpp"
self->setPrivate(CID_ASTTREE, new AstTreeHolder);
}

// list AstTree::getNodesInfo(){}
static QoreValue AstTree_getNodesInfo(QoreObject* self, AstTreeHolder* ath, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 62 "QC_AstTree.qpp"
    return ath->getNodesInfo();
}

// nothing AstTree::printTree(string file){}
static QoreValue AstTree_printTree_Vs(QoreObject* self, AstTreeHolder* ath, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* file = HARD_QORE_VALUE_STRING(args, 0);
# 53 "QC_AstTree.qpp"
    std::ofstream ofs(file->c_str(), std::ofstream::out | std::ofstream::trunc);
    ath->printTree(ofs);
    return QoreValue();
}

DLLLOCAL void preinitAstTreeClass() {
    QC_ASTTREE = new QoreClass("AstTree", "::astparser::AstTree", QDOM_DEFAULT);
    CID_ASTTREE = QC_ASTTREE->getID();
    QC_ASTTREE->setSystem();
}

DLLLOCAL QoreClass* initAstTreeClass(QoreNamespace& ns) {
    if (!QC_ASTTREE)
        preinitAstTreeClass();

    QC_ASTTREE->setFinal();

    // AstTree::constructor() {}
    QC_ASTTREE->addConstructor(AstTree_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // list AstTree::getNodesInfo(){}
    QC_ASTTREE->addMethod("getNodesInfo", (q_method_n_t)AstTree_getNodesInfo, Public, QCF_NO_FLAGS, QDOM_DEFAULT, listTypeInfo);

    // nothing AstTree::printTree(string file){}
    QC_ASTTREE->addMethod("printTree", (q_method_n_t)AstTree_printTree_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "file");

    return QC_ASTTREE;
}
