/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractConstant.cpp AbstractConstant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_AbstractConstant.h"
#include "QC_Class.h"
#include "QC_Type.h"
/* Qore class Qore::Reflection::AbstractConstant */

qore_classid_t CID_ABSTRACTCONSTANT;
QoreClass* QC_ABSTRACTCONSTANT;

// AbstractConstant::constructor() {}
static void AbstractConstant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 45 "QC_AbstractConstant.qpp"
xsink->raiseException("ABSTRACTCONSTANT-CONSTRUCTOR-ERROR", "this class cannot be constructed directly or inherited directly by a user-defined class");
}

// list<string> AbstractConstant::getModifierList(){}
static QoreValue AbstractConstant_getModifierList(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 160 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), xsink);
    const QoreClass* cls = c->getClass();
    if (cls) {
        rv->push(new QoreStringNode(get_access_string(c->ce->getAccess())), xsink);
    }
    else {
        if (c->ce->isModulePublic()) {
            rv->push(new QoreStringNode("public"), xsink);
        }
    }

    return rv.release();
}

// int AbstractConstant::getModifiers(){}
static QoreValue AbstractConstant_getModifiers(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 123 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    const QoreClass* cls = c->getClass();
    int64 rc = 0;
    if (cls) {
        rc |= get_access(c->ce->getAccess());
    }
    else {
        if (c->ce->isModulePublic()) {
            rc |= MC_PUBLIC;
        }
    }

    return rc;
}

// *string AbstractConstant::getModuleName(){}
static QoreValue AbstractConstant_getModuleName(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 80 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    const char* mod = c->ce->getModuleName();
    return mod ? new QoreStringNode(mod) : QoreValue();
}

// string AbstractConstant::getName(){}
static QoreValue AbstractConstant_getName(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 59 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    return new QoreStringNode(c->ce->getName());
}

// hash<SourceLocationInfo> AbstractConstant::getSourceLocation(){}
static QoreValue AbstractConstant_getSourceLocation(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 293 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    return c->ce->getSourceLocation()->getHash();
}

// Type AbstractConstant::getType(){}
static QoreValue AbstractConstant_getType(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 251 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return new QoreObject(QC_TYPE, c->pgm, new QoreType(c->ce->getTypeInfo()));
}

// auto AbstractConstant::getValue(){}
static QoreValue AbstractConstant_getValue(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 272 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    return c->ce->getReferencedValue();
}

// bool AbstractConstant::isBuiltin(){}
static QoreValue AbstractConstant_isBuiltin(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 211 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    return c->ce->isBuiltin();
}

// bool AbstractConstant::isClassConstant(){}
static QoreValue AbstractConstant_isClassConstant(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 192 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the constant ptr must be valid
    return (bool)c->getClass();
}

// bool AbstractConstant::isEqual(AbstractConstant con){}
static QoreValue AbstractConstant_isEqual_C16AbstractConstant(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(con, QoreReflectionConstant, args, 0, CID_ABSTRACTCONSTANT, "AbstractConstant::isEqual()", "AbstractConstant", xsink);
    if (*xsink)
        return 0;
# 102 "QC_AbstractConstant.qpp"
    ReferenceHolder<QoreReflectionConstant> holder(con, xsink);

    return con->ce == c->ce;
}

// bool AbstractConstant::isUser(){}
static QoreValue AbstractConstant_isUser(QoreObject* self, QoreReflectionConstant* c, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 230 "QC_AbstractConstant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, c->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return !c->ce->isBuiltin();
}

DLLLOCAL void preinitAbstractConstantClass() {
    QC_ABSTRACTCONSTANT = new QoreClass("AbstractConstant", "::Qore::Reflection::AbstractConstant", QDOM_REFLECTION);
    CID_ABSTRACTCONSTANT = QC_ABSTRACTCONSTANT->getID();
    QC_ABSTRACTCONSTANT->setSystem();
}

DLLLOCAL QoreClass* initAbstractConstantClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTCONSTANT)
        preinitAbstractConstantClass();

    // AbstractConstant::constructor() {}
    QC_ABSTRACTCONSTANT->addConstructor(AbstractConstant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // list<string> AbstractConstant::getModifierList(){}
    QC_ABSTRACTCONSTANT->addMethod("getModifierList", (q_method_n_t)AbstractConstant_getModifierList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int AbstractConstant::getModifiers(){}
    QC_ABSTRACTCONSTANT->addMethod("getModifiers", (q_method_n_t)AbstractConstant_getModifiers, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // *string AbstractConstant::getModuleName(){}
    QC_ABSTRACTCONSTANT->addMethod("getModuleName", (q_method_n_t)AbstractConstant_getModuleName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string AbstractConstant::getName(){}
    QC_ABSTRACTCONSTANT->addMethod("getName", (q_method_n_t)AbstractConstant_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // hash<SourceLocationInfo> AbstractConstant::getSourceLocation(){}
    QC_ABSTRACTCONSTANT->addMethod("getSourceLocation", (q_method_n_t)AbstractConstant_getSourceLocation, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, hashdeclSourceLocationInfo->getTypeInfo(false));

    // Type AbstractConstant::getType(){}
    QC_ABSTRACTCONSTANT->addMethod("getType", (q_method_n_t)AbstractConstant_getType, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // auto AbstractConstant::getValue(){}
    QC_ABSTRACTCONSTANT->addMethod("getValue", (q_method_n_t)AbstractConstant_getValue, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo);

    // bool AbstractConstant::isBuiltin(){}
    QC_ABSTRACTCONSTANT->addMethod("isBuiltin", (q_method_n_t)AbstractConstant_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractConstant::isClassConstant(){}
    QC_ABSTRACTCONSTANT->addMethod("isClassConstant", (q_method_n_t)AbstractConstant_isClassConstant, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractConstant::isEqual(AbstractConstant con){}
    QC_ABSTRACTCONSTANT->addMethod("isEqual", (q_method_n_t)AbstractConstant_isEqual_C16AbstractConstant, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_ABSTRACTCONSTANT->getTypeInfo(), QORE_PARAM_NO_ARG, "con");

    // bool AbstractConstant::isUser(){}
    QC_ABSTRACTCONSTANT->addMethod("isUser", (q_method_n_t)AbstractConstant_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    return QC_ABSTRACTCONSTANT;
}
