/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_FunctionVariant.cpp FunctionVariant class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_FunctionVariant.h"
/* Qore class Qore::Reflection::FunctionVariant */

qore_classid_t CID_FUNCTIONVARIANT;
QoreClass* QC_FUNCTIONVARIANT;

// auto FunctionVariant::call(...){}
static QoreValue FunctionVariant_call_VV(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 67 "QC_FunctionVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->f->evalFunction(v->variant, args, v->pgm, xsink);
}

// auto FunctionVariant::callArgs(*softlist<auto> argv){}
static QoreValue FunctionVariant_callArgs_C15_softlist_auto_(QoreObject* self, QoreReflectionVariant* v, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* argv = get_param_value(args, 0).get<const QoreListNode>();
# 93 "QC_FunctionVariant.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, v->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the variant ptr must be valid
    return v->f->evalFunction(v->variant, argv, v->pgm, xsink);
}

// FunctionVariant::constructor() {}
static void FunctionVariant_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 48 "QC_FunctionVariant.qpp"
xsink->raiseException("FUNCTIONVARIANT-CONSTRUCTOR-ERROR", "this class is an abstract class and cannot be constructed directly or inherited directly by a user-defined class");
}

DLLLOCAL void preinitFunctionVariantClass() {
    QC_FUNCTIONVARIANT = new QoreClass("FunctionVariant", "::Qore::Reflection::FunctionVariant", QDOM_REFLECTION);
    CID_FUNCTIONVARIANT = QC_FUNCTIONVARIANT->getID();
    QC_FUNCTIONVARIANT->setSystem();
}

DLLLOCAL QoreClass* initFunctionVariantClass(QoreNamespace& ns) {
    if (!QC_FUNCTIONVARIANT)
        preinitFunctionVariantClass();

    // set parent class
    assert(QC_ABSTRACTVARIANT);
    QC_FUNCTIONVARIANT->addBuiltinVirtualBaseClass(QC_ABSTRACTVARIANT);

    // auto FunctionVariant::call(...){}
    QC_FUNCTIONVARIANT->addMethod("call", (q_method_n_t)FunctionVariant_call_VV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);

    // auto FunctionVariant::callArgs(*softlist<auto> argv){}
    QC_FUNCTIONVARIANT->addMethod("callArgs", (q_method_n_t)FunctionVariant_callArgs_C15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "argv");

    // FunctionVariant::constructor() {}
    QC_FUNCTIONVARIANT->addConstructor(FunctionVariant_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    return QC_FUNCTIONVARIANT;
}
